# ruff: noqa: E501, PLR0915
from __future__ import annotations

types: dict[str, str] = {
    "AWS::ACMPCA::Certificate": "8095f49d3b9d3b1a",
    "AWS::ACMPCA::CertificateAuthority": "aaf234b2120b5659",
    "AWS::ACMPCA::CertificateAuthorityActivation": "a62fb980b49a2961",
    "AWS::ACMPCA::Permission": "04b877e245730b3e",
    "AWS::APS::AnomalyDetector": "ce9db55fb4165695",
    "AWS::APS::ResourcePolicy": "da0d17341b025c5d",
    "AWS::APS::RuleGroupsNamespace": "1898bd2e1a2fc3af",
    "AWS::APS::Scraper": "6d167939ca161a78",
    "AWS::APS::Workspace": "0739c3a13c4ccac6",
    "AWS::ARCRegionSwitch::Plan": "a1a2ea55c2fbda5a",
    "AWS::ARCZonalShift::AutoshiftObserverNotificationStatus": "806adfa6e5aba057",
    "AWS::ARCZonalShift::ZonalAutoshiftConfiguration": "a2125021f0082633",
    "AWS::AccessAnalyzer::Analyzer": "165f750ca8d78dde",
    "AWS::AmazonMQ::Broker": "c7ad1fe2fbeb2d1b",
    "AWS::AmazonMQ::Configuration": "9a9a64561c801ccb",
    "AWS::AmazonMQ::ConfigurationAssociation": "2343e0ffe1114046",
    "AWS::ApiGateway::Account": "1876f63604439391",
    "AWS::ApiGateway::ApiKey": "48205d4ffd7feba3",
    "AWS::ApiGateway::Authorizer": "2ace6030ba917092",
    "AWS::ApiGateway::BasePathMapping": "14bcf4ff90345f78",
    "AWS::ApiGateway::BasePathMappingV2": "361e364ef63cf6fd",
    "AWS::ApiGateway::ClientCertificate": "3c72d768c43d41a3",
    "AWS::ApiGateway::Deployment": "2ffb8f3345f190c2",
    "AWS::ApiGateway::DocumentationPart": "1383fef137de858b",
    "AWS::ApiGateway::DocumentationVersion": "8f19f440f5e47b9f",
    "AWS::ApiGateway::DomainName": "1b67859ef84a0247",
    "AWS::ApiGateway::DomainNameAccessAssociation": "a990644bdd54299e",
    "AWS::ApiGateway::DomainNameV2": "a0369b9dd79fc7c8",
    "AWS::ApiGateway::GatewayResponse": "1ee3349fb55bdd49",
    "AWS::ApiGateway::Method": "8d3568941b9c1cfe",
    "AWS::ApiGateway::Model": "80f210a42639b333",
    "AWS::ApiGateway::RequestValidator": "afc1110de614be93",
    "AWS::ApiGateway::Resource": "0b20e16c9c975c0d",
    "AWS::ApiGateway::RestApi": "b507ba6256e35f11",
    "AWS::ApiGateway::Stage": "56ecb180a441ee32",
    "AWS::ApiGateway::UsagePlan": "c9548a0a281386e5",
    "AWS::ApiGateway::UsagePlanKey": "309b26469128fd0a",
    "AWS::ApiGateway::VpcLink": "3c3c844926fba248",
    "AWS::ApiGatewayV2::Api": "fdc84407d8d6260e",
    "AWS::ApiGatewayV2::ApiGatewayManagedOverrides": "6852b176617879b0",
    "AWS::ApiGatewayV2::ApiMapping": "907de0790553e448",
    "AWS::ApiGatewayV2::Authorizer": "69107e1e01673765",
    "AWS::ApiGatewayV2::Deployment": "81174b55f308dad3",
    "AWS::ApiGatewayV2::DomainName": "87535a11b8340a18",
    "AWS::ApiGatewayV2::Integration": "77743451e77a7545",
    "AWS::ApiGatewayV2::IntegrationResponse": "a319db9e828555fa",
    "AWS::ApiGatewayV2::Model": "fcc09a7d8ffbd09c",
    "AWS::ApiGatewayV2::Route": "112b036b7d923f13",
    "AWS::ApiGatewayV2::RouteResponse": "3606f855415b8421",
    "AWS::ApiGatewayV2::RoutingRule": "b9e023c3c57c3c04",
    "AWS::ApiGatewayV2::Stage": "2903f59facc01e1c",
    "AWS::ApiGatewayV2::VpcLink": "e5ea6845d4e71e0b",
    "AWS::AppConfig::Application": "a84c56270dc5bf5d",
    "AWS::AppConfig::ConfigurationProfile": "d44f95228eb08d3c",
    "AWS::AppConfig::Deployment": "7a861ec5d43b4903",
    "AWS::AppConfig::DeploymentStrategy": "034671c91cf2b614",
    "AWS::AppConfig::Environment": "0c307beb222c782d",
    "AWS::AppConfig::Extension": "1c79fa5b4d27bc9a",
    "AWS::AppConfig::ExtensionAssociation": "85102348a3d001d3",
    "AWS::AppConfig::HostedConfigurationVersion": "4d2f292068fae1c6",
    "AWS::AppSync::Api": "b46594583c264d17",
    "AWS::AppSync::ApiCache": "d7a7fb82edccdfe2",
    "AWS::AppSync::ApiKey": "3d104ea0be48564b",
    "AWS::AppSync::ChannelNamespace": "194048f8c8383a17",
    "AWS::AppSync::DataSource": "874f010f3099ba5c",
    "AWS::AppSync::DomainName": "90cf6b5b0582fe8c",
    "AWS::AppSync::DomainNameApiAssociation": "d4af4b8aef39897a",
    "AWS::AppSync::FunctionConfiguration": "f0c6eaeaf8181363",
    "AWS::AppSync::GraphQLApi": "9f7a289328001b58",
    "AWS::AppSync::GraphQLSchema": "b3409083f48d1b5b",
    "AWS::AppSync::Resolver": "f55e989a04fef713",
    "AWS::AppSync::SourceApiAssociation": "8176278e728ac6a7",
    "AWS::ApplicationAutoScaling::ScalableTarget": "de001f7e2b117e91",
    "AWS::ApplicationAutoScaling::ScalingPolicy": "e448f1493d81f461",
    "AWS::ApplicationInsights::Application": "4fb681541bc69d81",
    "AWS::ApplicationSignals::Discovery": "2bbcdf020ea7fc15",
    "AWS::ApplicationSignals::GroupingConfiguration": "d6ba1088512bf212",
    "AWS::ApplicationSignals::ServiceLevelObjective": "148bbf048d668735",
    "AWS::Athena::CapacityReservation": "cf670cf0253b02eb",
    "AWS::Athena::DataCatalog": "72b7e9ce721875ee",
    "AWS::Athena::NamedQuery": "1fd9b221f87bab5d",
    "AWS::Athena::PreparedStatement": "efe18f0108881514",
    "AWS::Athena::WorkGroup": "98125aa523006e5f",
    "AWS::AutoScaling::AutoScalingGroup": "765a79d282c7736e",
    "AWS::AutoScaling::LaunchConfiguration": "692d8c3e4a5cb93c",
    "AWS::AutoScaling::LifecycleHook": "0ee139c35778edce",
    "AWS::AutoScaling::ScalingPolicy": "68daf421ab1876df",
    "AWS::AutoScaling::ScheduledAction": "ab00e084f1d921e6",
    "AWS::AutoScaling::WarmPool": "52a6245a933ab185",
    "AWS::Backup::BackupPlan": "59655b37955d7bc9",
    "AWS::Backup::BackupSelection": "26040166156c899a",
    "AWS::Backup::BackupVault": "695e80a4aaec9275",
    "AWS::Backup::Framework": "227a7717971e8103",
    "AWS::Backup::LogicallyAirGappedBackupVault": "99976d61395f41e3",
    "AWS::Backup::ReportPlan": "ac5222585ea4fb04",
    "AWS::Backup::RestoreTestingPlan": "a1fd4f431c67c9f4",
    "AWS::Backup::RestoreTestingSelection": "d4f6fc7c7d230fb2",
    "AWS::Backup::TieringConfiguration": "f16772cf7bc87fe1",
    "AWS::Batch::ComputeEnvironment": "90bfbf164c530f34",
    "AWS::Batch::ConsumableResource": "8e12226640545a9a",
    "AWS::Batch::JobDefinition": "8a7ecae4430d86b2",
    "AWS::Batch::JobQueue": "330cfa7e67eefadc",
    "AWS::Batch::QuotaShare": "8243d68bc7fae554",
    "AWS::Batch::SchedulingPolicy": "2421cebc8e80f036",
    "AWS::Batch::ServiceEnvironment": "72c42b7d08edd4dc",
    "AWS::Bedrock::Agent": "b9208a8b61851203",
    "AWS::Bedrock::AgentAlias": "957e2ce5a7007785",
    "AWS::Bedrock::DataSource": "9ca916388ec8b805",
    "AWS::Bedrock::Flow": "3d47cd9b2b799711",
    "AWS::Bedrock::FlowAlias": "36fa0dff459bcfd0",
    "AWS::Bedrock::FlowVersion": "3f011854b48e4c5b",
    "AWS::Bedrock::Guardrail": "2cb3f0e888718dad",
    "AWS::Bedrock::GuardrailVersion": "5bf27647412382c4",
    "AWS::Bedrock::KnowledgeBase": "42afb318cede00fc",
    "AWS::Bedrock::Prompt": "63cb5b7f989bfc36",
    "AWS::Bedrock::PromptVersion": "41e3e79078ba156a",
    "AWS::CDK::Metadata": "6c85ec990b9ac854",
    "AWS::CE::AnomalyMonitor": "0da3e799de4393c6",
    "AWS::CE::AnomalySubscription": "626bad41898c9c78",
    "AWS::CertificateManager::Certificate": "215c8ed2f4fd8b1d",
    "AWS::Chatbot::CustomAction": "8dfc85a2f8e2f05e",
    "AWS::Chatbot::MicrosoftTeamsChannelConfiguration": "010fe053ff4053d8",
    "AWS::Chatbot::SlackChannelConfiguration": "733a3e7b4bdf0053",
    "AWS::CloudFormation::CustomResource": "e1f552bc7275a696",
    "AWS::CloudFormation::GuardHook": "a48f290f69a60c20",
    "AWS::CloudFormation::HookDefaultVersion": "ed13bf5a5bcf4773",
    "AWS::CloudFormation::HookTypeConfig": "8dac7817ef160a80",
    "AWS::CloudFormation::HookVersion": "d23903bd79b22868",
    "AWS::CloudFormation::LambdaHook": "8e773b451512bd0f",
    "AWS::CloudFormation::Macro": "8517053591f9eceb",
    "AWS::CloudFormation::PublicTypeVersion": "e68574e847dfa548",
    "AWS::CloudFormation::Publisher": "03c41096bbdb2ba7",
    "AWS::CloudFormation::ResourceDefaultVersion": "f614137749ff39d3",
    "AWS::CloudFormation::ResourceVersion": "50aebd11bbf9830b",
    "AWS::CloudFormation::Stack": "089169583d0e826d",
    "AWS::CloudFormation::StackSet": "38258994c717eada",
    "AWS::CloudFormation::TypeActivation": "6e3bec3534fc9f42",
    "AWS::CloudFormation::WaitCondition": "6c5dfc2296cd28df",
    "AWS::CloudFormation::WaitConditionHandle": "edba290fd70976a3",
    "AWS::CloudFront::AnycastIpList": "beb2df4a9b037d07",
    "AWS::CloudFront::CachePolicy": "53d9f632b7e1d5b7",
    "AWS::CloudFront::CloudFrontOriginAccessIdentity": "d2e8d397beac5797",
    "AWS::CloudFront::ConnectionFunction": "45b04e615a243359",
    "AWS::CloudFront::ConnectionGroup": "68ca2c6e67355a40",
    "AWS::CloudFront::ContinuousDeploymentPolicy": "bf0fd1f8e692b34d",
    "AWS::CloudFront::Distribution": "6d7cb47a3edf25ac",
    "AWS::CloudFront::DistributionTenant": "d040de3fcc228c8d",
    "AWS::CloudFront::Function": "884b82a224857dd2",
    "AWS::CloudFront::KeyGroup": "7a9f609d809f1f8e",
    "AWS::CloudFront::KeyValueStore": "3cecb676246af3f9",
    "AWS::CloudFront::MonitoringSubscription": "f9dae15a96e422a7",
    "AWS::CloudFront::OriginAccessControl": "17f386694e5709fe",
    "AWS::CloudFront::OriginRequestPolicy": "97711158d1bc4353",
    "AWS::CloudFront::PublicKey": "b5cac84e7e953094",
    "AWS::CloudFront::RealtimeLogConfig": "94564597650a9bf8",
    "AWS::CloudFront::ResponseHeadersPolicy": "d5eaaa7cf7b104dc",
    "AWS::CloudFront::TrustStore": "52949233848f0e48",
    "AWS::CloudFront::VpcOrigin": "eed344f918658fdd",
    "AWS::CloudTrail::Dashboard": "28e73ef34409d386",
    "AWS::CloudTrail::EventDataStore": "a51aca6f15d716e3",
    "AWS::CloudTrail::ResourcePolicy": "ab2ae9ccbbd62450",
    "AWS::CloudTrail::Trail": "7afdcfb6fa95406a",
    "AWS::CloudWatch::Alarm": "ad61b62f1dc21f36",
    "AWS::CloudWatch::AlarmMuteRule": "8ae4572f5b52e53d",
    "AWS::CloudWatch::AnomalyDetector": "13b8b743191bf50f",
    "AWS::CloudWatch::CompositeAlarm": "2b2200aed34aabc4",
    "AWS::CloudWatch::Dashboard": "423086b5b02bd126",
    "AWS::CloudWatch::InsightRule": "4ac6b97801b575bd",
    "AWS::CloudWatch::MetricStream": "ab20b9e49eb444d9",
    "AWS::CodeBuild::Project": "1da142422f2ef55c",
    "AWS::CodeBuild::ReportGroup": "b34ce85ee1e4a8da",
    "AWS::CodeBuild::SourceCredential": "b84e190d9ebbbfe1",
    "AWS::CodeDeploy::Application": "eea8100bd279b205",
    "AWS::CodeDeploy::DeploymentConfig": "394f39ae21382747",
    "AWS::CodeDeploy::DeploymentGroup": "a5b18163cfa68275",
    "AWS::CodePipeline::CustomActionType": "b1ae4c1b735eeb9f",
    "AWS::CodePipeline::Pipeline": "ecdbe888f27625fb",
    "AWS::CodePipeline::Webhook": "a6ad2a4dc85ed032",
    "AWS::Cognito::IdentityPool": "3df22057c8e2ef54",
    "AWS::Cognito::IdentityPoolPrincipalTag": "3ae793569104c4f6",
    "AWS::Cognito::IdentityPoolRoleAttachment": "2a5296f4ca5a2ff7",
    "AWS::Cognito::LogDeliveryConfiguration": "2e1fa665bc591a0b",
    "AWS::Cognito::ManagedLoginBranding": "5c891e6d32f0ba8d",
    "AWS::Cognito::Terms": "f5bf1f3a97d979ba",
    "AWS::Cognito::UserPool": "615b9da8e393791f",
    "AWS::Cognito::UserPoolClient": "fe665a1921d29144",
    "AWS::Cognito::UserPoolDomain": "b89172fa4b1cf22a",
    "AWS::Cognito::UserPoolGroup": "a7315a389c858da9",
    "AWS::Cognito::UserPoolIdentityProvider": "497642110bee02ad",
    "AWS::Cognito::UserPoolResourceServer": "89ba39a21f9326df",
    "AWS::Cognito::UserPoolRiskConfigurationAttachment": "650fce2645198ef2",
    "AWS::Cognito::UserPoolUICustomizationAttachment": "9a793e05ae357b1c",
    "AWS::Cognito::UserPoolUser": "61780cf8beb842dc",
    "AWS::Cognito::UserPoolUserToGroupAttachment": "642c498727b1c8f4",
    "AWS::Config::AggregationAuthorization": "af318b77aaceeeaa",
    "AWS::Config::ConfigRule": "907ef12e35734983",
    "AWS::Config::ConfigurationAggregator": "6d45617d9786aeb6",
    "AWS::Config::ConfigurationRecorder": "e35cee148e38b939",
    "AWS::Config::ConformancePack": "4fa6a17df94cbade",
    "AWS::Config::DeliveryChannel": "4f7af4ff91e52582",
    "AWS::Config::OrganizationConfigRule": "1f49966ab4677d68",
    "AWS::Config::OrganizationConformancePack": "bc4a95ffdb9c3942",
    "AWS::Config::StoredQuery": "12f2c4b9dcad23d7",
    "AWS::ControlTower::EnabledBaseline": "fc45dfb83aa6dd82",
    "AWS::ControlTower::EnabledControl": "ac0e492fde8853d9",
    "AWS::ControlTower::LandingZone": "96c88ada196eeb0e",
    "AWS::DLM::LifecyclePolicy": "1bf383e7f1ad322d",
    "AWS::DMS::DataMigration": "ade967581e1dfc34",
    "AWS::DMS::DataProvider": "94c68c86840900fc",
    "AWS::DMS::InstanceProfile": "b289107706553b44",
    "AWS::DMS::MigrationProject": "b4b035d4b0f0b861",
    "AWS::DMS::ReplicationConfig": "563e9b573376b7ec",
    "AWS::DataPipeline::Pipeline": "0200bdbc5529afe3",
    "AWS::DataSync::Agent": "d3a5fe3ab751b18f",
    "AWS::DataSync::LocationAzureBlob": "6d28fbfcade96028",
    "AWS::DataSync::LocationEFS": "88ace952f3e3b739",
    "AWS::DataSync::LocationFSxLustre": "38fcf6001961bbfc",
    "AWS::DataSync::LocationFSxONTAP": "d022ea0462e623a1",
    "AWS::DataSync::LocationFSxWindows": "db12cfe074e88d2d",
    "AWS::DataSync::LocationHDFS": "5ef399d200cb55db",
    "AWS::DataSync::LocationNFS": "fd57f33b0f494c27",
    "AWS::DataSync::LocationObjectStorage": "a5012c826a08f3e8",
    "AWS::DataSync::LocationS3": "d9f087e59be69eec",
    "AWS::DataSync::LocationSMB": "01bbd3daa55f2a4c",
    "AWS::DataSync::Task": "983b864808e1db63",
    "AWS::DirectConnect::Connection": "816b9e564bb15a2d",
    "AWS::DirectConnect::DirectConnectGateway": "064ba92b6e29e1bb",
    "AWS::DirectConnect::DirectConnectGatewayAssociation": "e8b6eb4a71af1eee",
    "AWS::DirectConnect::Lag": "5d6f47c1a9673feb",
    "AWS::DirectConnect::PrivateVirtualInterface": "6c8c1e1388789e05",
    "AWS::DirectConnect::PublicVirtualInterface": "ac13a4f01149c397",
    "AWS::DirectConnect::TransitVirtualInterface": "4cbdabc6ea22d6c4",
    "AWS::DirectoryService::MicrosoftAD": "7a48b0e737a0833b",
    "AWS::DirectoryService::SimpleAD": "a87f6c61453bc19b",
    "AWS::DocDB::DBCluster": "7e5b3d7a431be20d",
    "AWS::DocDB::DBClusterParameterGroup": "3e017448a15e0f15",
    "AWS::DocDB::DBInstance": "253562f270b587ac",
    "AWS::DocDB::DBSubnetGroup": "ccb97a9452b2284d",
    "AWS::DocDB::EventSubscription": "d22815028d03f301",
    "AWS::DocDB::GlobalCluster": "6cbd1fec2debea5e",
    "AWS::DynamoDB::GlobalTable": "06d89d7f61a3db96",
    "AWS::DynamoDB::Table": "e2d8737a0d206fda",
    "AWS::EC2::CapacityReservation": "1ab29c6314095182",
    "AWS::EC2::CapacityReservationFleet": "66f529dd9ae70683",
    "AWS::EC2::ClientVpnAuthorizationRule": "09dec539b3c3f428",
    "AWS::EC2::ClientVpnEndpoint": "d132f32844d7e05e",
    "AWS::EC2::ClientVpnRoute": "f1d130776f4e1473",
    "AWS::EC2::ClientVpnTargetNetworkAssociation": "d69233d655faf0f6",
    "AWS::EC2::CustomerGateway": "4003177e29b1dfc5",
    "AWS::EC2::DHCPOptions": "7794e3a54eaacb7a",
    "AWS::EC2::EC2Fleet": "3452719ae9095492",
    "AWS::EC2::EIP": "cc42df82cdc81e9c",
    "AWS::EC2::EIPAssociation": "a267820d6250f189",
    "AWS::EC2::EgressOnlyInternetGateway": "922c59e19b698eac",
    "AWS::EC2::FlowLog": "ea553ae6f4a71ce3",
    "AWS::EC2::GatewayRouteTableAssociation": "50b450990964d2e7",
    "AWS::EC2::Host": "078ff42d61bf90e9",
    "AWS::EC2::IPAM": "3bab32d28a4ef063",
    "AWS::EC2::IPAMAllocation": "c4cba824ca02ce4f",
    "AWS::EC2::IPAMPool": "f8632a39914b3366",
    "AWS::EC2::IPAMPoolCidr": "873e7cc6e6afef96",
    "AWS::EC2::IPAMPrefixListResolver": "475d81b7a683c9d6",
    "AWS::EC2::IPAMPrefixListResolverTarget": "6c1937197e4b2351",
    "AWS::EC2::IPAMResourceDiscovery": "0a21294934d111b5",
    "AWS::EC2::IPAMResourceDiscoveryAssociation": "fde93486402ddbe5",
    "AWS::EC2::IPAMScope": "f1341090e0e57749",
    "AWS::EC2::Instance": "a61701d41fad1efa",
    "AWS::EC2::InstanceConnectEndpoint": "bef7bacc666f5402",
    "AWS::EC2::InternetGateway": "4c7c90cb3105c9d8",
    "AWS::EC2::IpPoolRouteTableAssociation": "071b5a1f00936f72",
    "AWS::EC2::KeyPair": "349deb28f79434e4",
    "AWS::EC2::LaunchTemplate": "dbb4de05d9ee7263",
    "AWS::EC2::NatGateway": "28e80cb1fcf92145",
    "AWS::EC2::NetworkAcl": "41c7ec1cea436cd8",
    "AWS::EC2::NetworkAclEntry": "d4fc9ce8669023f0",
    "AWS::EC2::NetworkInsightsAccessScope": "f174ef9549ba3fd9",
    "AWS::EC2::NetworkInsightsAccessScopeAnalysis": "d9d932b88ae8cef7",
    "AWS::EC2::NetworkInsightsAnalysis": "594bd95d5c037217",
    "AWS::EC2::NetworkInsightsPath": "98fa1559e7adbeec",
    "AWS::EC2::NetworkInterface": "1e436db5d46542c6",
    "AWS::EC2::NetworkInterfaceAttachment": "0d96c6d7269bdfb0",
    "AWS::EC2::NetworkInterfacePermission": "1f44f2ab1e0b19cb",
    "AWS::EC2::NetworkPerformanceMetricSubscription": "a507c085bf7ffbcf",
    "AWS::EC2::PlacementGroup": "8dca9ce7eb8b50ac",
    "AWS::EC2::PrefixList": "f956982a96339c09",
    "AWS::EC2::Route": "0ff811157612151a",
    "AWS::EC2::RouteServer": "f53338d8b3c6499a",
    "AWS::EC2::RouteServerAssociation": "22bb952c9a61f2a9",
    "AWS::EC2::RouteServerEndpoint": "40a2b7461aa985fa",
    "AWS::EC2::RouteServerPeer": "0b52f45b8ea8cbac",
    "AWS::EC2::RouteServerPropagation": "0dd3a42d71c85b11",
    "AWS::EC2::RouteTable": "67bdbfcad069c66a",
    "AWS::EC2::SecurityGroup": "5390282384be7866",
    "AWS::EC2::SecurityGroupEgress": "840df778de9b4be2",
    "AWS::EC2::SecurityGroupIngress": "367136b2f03f428c",
    "AWS::EC2::SecurityGroupVpcAssociation": "f48680388ee894fe",
    "AWS::EC2::SnapshotBlockPublicAccess": "f6434397a8c7e382",
    "AWS::EC2::SpotFleet": "b3c6b5b2f4a5b7d2",
    "AWS::EC2::SqlHaStandbyDetectedInstance": "9cf7ff753948e0b7",
    "AWS::EC2::Subnet": "a60b8dc0c7f50396",
    "AWS::EC2::SubnetCidrBlock": "473785ce782ba18f",
    "AWS::EC2::SubnetNetworkAclAssociation": "6b4f4042734e67b7",
    "AWS::EC2::SubnetRouteTableAssociation": "6b18db45c4ef5f57",
    "AWS::EC2::TrafficMirrorFilter": "d97085902d4e54ae",
    "AWS::EC2::TrafficMirrorFilterRule": "df0dc3904abb859c",
    "AWS::EC2::TrafficMirrorSession": "ec54e16ff79eaac0",
    "AWS::EC2::TrafficMirrorTarget": "562f502a60abccd6",
    "AWS::EC2::TransitGateway": "bdd0278ecaccd4ae",
    "AWS::EC2::TransitGatewayAttachment": "40111e00bf0df74a",
    "AWS::EC2::TransitGatewayConnect": "f3001456d8e71829",
    "AWS::EC2::TransitGatewayConnectPeer": "27459a628d254042",
    "AWS::EC2::TransitGatewayMeteringPolicy": "4daf1939996f77df",
    "AWS::EC2::TransitGatewayMeteringPolicyEntry": "49afaade66d17b06",
    "AWS::EC2::TransitGatewayMulticastDomain": "55ac3f0e32f72ab4",
    "AWS::EC2::TransitGatewayMulticastDomainAssociation": "f650430e0bbdcce9",
    "AWS::EC2::TransitGatewayMulticastGroupMember": "fe8c827e156dc451",
    "AWS::EC2::TransitGatewayMulticastGroupSource": "95b20156fbc77eb0",
    "AWS::EC2::TransitGatewayPeeringAttachment": "2a33a8efb207e5e5",
    "AWS::EC2::TransitGatewayRoute": "41a8efed021fd377",
    "AWS::EC2::TransitGatewayRouteTable": "cb8a1d0ac988c165",
    "AWS::EC2::TransitGatewayRouteTableAssociation": "a28bde73d4a5cde6",
    "AWS::EC2::TransitGatewayRouteTablePropagation": "e21e0c3273d49afe",
    "AWS::EC2::TransitGatewayVpcAttachment": "9f86e47a59b89462",
    "AWS::EC2::VPC": "f08626e9e82df64f",
    "AWS::EC2::VPCBlockPublicAccessExclusion": "27a1b09ed6dc90f2",
    "AWS::EC2::VPCBlockPublicAccessOptions": "1227e027d1c429f9",
    "AWS::EC2::VPCCidrBlock": "2ab876850f9f6c40",
    "AWS::EC2::VPCDHCPOptionsAssociation": "f803bdbad7d930d1",
    "AWS::EC2::VPCEncryptionControl": "0dff101542854d6c",
    "AWS::EC2::VPCEndpoint": "1ece9f120d9da74a",
    "AWS::EC2::VPCEndpointConnectionNotification": "4d7f63de5d4c3a2e",
    "AWS::EC2::VPCEndpointService": "150eafe6174fc552",
    "AWS::EC2::VPCEndpointServicePermissions": "5841ea22a0cbb143",
    "AWS::EC2::VPCGatewayAttachment": "1dc77043848e8e94",
    "AWS::EC2::VPCPeeringConnection": "41b9860245304058",
    "AWS::EC2::VPNConcentrator": "78aca3962b6da301",
    "AWS::EC2::VPNConnection": "64de71e4f82fffe6",
    "AWS::EC2::VPNConnectionRoute": "00194e67a2d7f53d",
    "AWS::EC2::VPNGateway": "1e2f029e99547e1a",
    "AWS::EC2::VPNGatewayRoutePropagation": "5e03d41fa9dc15b9",
    "AWS::EC2::Volume": "3385669f1c48cc6c",
    "AWS::EC2::VolumeAttachment": "1f1ab62c0d6b0fdb",
    "AWS::ECR::PullThroughCacheRule": "d098a5f375d1672b",
    "AWS::ECR::PullTimeUpdateExclusion": "67ad05acb3cd4d21",
    "AWS::ECR::RegistryPolicy": "a36a04f4454cafc2",
    "AWS::ECR::RegistryScanningConfiguration": "68012043447b437a",
    "AWS::ECR::ReplicationConfiguration": "dc2a377f75525dd4",
    "AWS::ECR::Repository": "f436b8d080c711a7",
    "AWS::ECR::RepositoryCreationTemplate": "53b4234f411273ec",
    "AWS::ECS::CapacityProvider": "7319d435dadca230",
    "AWS::ECS::Cluster": "92b09b679d42fc1e",
    "AWS::ECS::ClusterCapacityProviderAssociations": "4b512d1fe30ffdd3",
    "AWS::ECS::ExpressGatewayService": "03b7c9db813057d9",
    "AWS::ECS::PrimaryTaskSet": "d8ea9e906cb85b80",
    "AWS::ECS::Service": "15d70b98c4d7b699",
    "AWS::ECS::TaskDefinition": "b9f3cf615fa7a616",
    "AWS::ECS::TaskSet": "468acfd59cd44c08",
    "AWS::EFS::AccessPoint": "48e898365c908cfd",
    "AWS::EFS::FileSystem": "575ef4f21bc6f9da",
    "AWS::EFS::MountTarget": "562530cf9ce264a6",
    "AWS::EKS::AccessEntry": "6eb6ba186799e7f3",
    "AWS::EKS::Addon": "63cae0a921c6a93d",
    "AWS::EKS::Capability": "7816d4a96b146b20",
    "AWS::EKS::Cluster": "b99bad48868ee64a",
    "AWS::EKS::FargateProfile": "54a9efe57f089a05",
    "AWS::EKS::IdentityProviderConfig": "77b807192827540b",
    "AWS::EKS::Nodegroup": "11f90e70c2b0b5a1",
    "AWS::EKS::PodIdentityAssociation": "c9fa86f74de95001",
    "AWS::EMR::Cluster": "ac0466c8197ef416",
    "AWS::EMR::InstanceFleetConfig": "7c6308bb9ea61ce8",
    "AWS::EMR::InstanceGroupConfig": "57ea4fa9b7b6f658",
    "AWS::EMR::SecurityConfiguration": "63bc4abbfd8348f4",
    "AWS::EMR::Step": "b5416f76c2b6970c",
    "AWS::EMR::WALWorkspace": "1acf5569fff4fcbe",
    "AWS::EMRContainers::VirtualCluster": "d56c1799bc259539",
    "AWS::EMRServerless::Application": "ebe59c5ecbcc031b",
    "AWS::EVS::Environment": "de40f0a1095cfeb8",
    "AWS::ElastiCache::CacheCluster": "1220b6bbd633ae68",
    "AWS::ElastiCache::GlobalReplicationGroup": "2af3062bf5b2cf7f",
    "AWS::ElastiCache::ParameterGroup": "b0e1dec3cd05f739",
    "AWS::ElastiCache::ReplicationGroup": "60747209a9932d90",
    "AWS::ElastiCache::SecurityGroup": "075dfa59da275723",
    "AWS::ElastiCache::SecurityGroupIngress": "96d1b4a5bd438d25",
    "AWS::ElastiCache::ServerlessCache": "67ad14ea2425e853",
    "AWS::ElastiCache::SubnetGroup": "b331f1d336ff42bf",
    "AWS::ElastiCache::User": "e1de3c410639ffec",
    "AWS::ElastiCache::UserGroup": "13de0814cb27012a",
    "AWS::ElasticBeanstalk::Application": "53e8948ce5a8b24c",
    "AWS::ElasticBeanstalk::ApplicationVersion": "c046361d8f74f1d2",
    "AWS::ElasticBeanstalk::ConfigurationTemplate": "0c562f3a39a4bf77",
    "AWS::ElasticBeanstalk::Environment": "c4d0bd36985e69d1",
    "AWS::ElasticLoadBalancing::LoadBalancer": "0c104e32be4cf97e",
    "AWS::ElasticLoadBalancingV2::Listener": "c812f53b913442e6",
    "AWS::ElasticLoadBalancingV2::ListenerCertificate": "3edd898ae1082797",
    "AWS::ElasticLoadBalancingV2::ListenerRule": "1887769bb7b98082",
    "AWS::ElasticLoadBalancingV2::LoadBalancer": "a0c22c95cd0559a6",
    "AWS::ElasticLoadBalancingV2::TargetGroup": "e0f0e8c5589655c1",
    "AWS::ElasticLoadBalancingV2::TrustStore": "91b22a729d45fd13",
    "AWS::ElasticLoadBalancingV2::TrustStoreRevocation": "593a78e5ca9c2be3",
    "AWS::Elasticsearch::Domain": "c6414ef13f2fedbe",
    "AWS::Events::ApiDestination": "7c2cdd49d75e6a5d",
    "AWS::Events::Archive": "6003bd8f99a7904a",
    "AWS::Events::Connection": "eefba33e4beba85e",
    "AWS::Events::EventBus": "ef510d33dcc074ea",
    "AWS::Events::EventBusPolicy": "16de109949ef16e2",
    "AWS::Events::Rule": "65aec73d3b811c08",
    "AWS::FMS::NotificationChannel": "3dfde465b38d83be",
    "AWS::FMS::Policy": "9a5d16a686ed3b5d",
    "AWS::FMS::ResourceSet": "6fb04db2dcf50bf8",
    "AWS::FSx::DataRepositoryAssociation": "ecfa3f43f4917224",
    "AWS::FSx::FileSystem": "de008ab78c1f98a8",
    "AWS::FSx::S3AccessPointAttachment": "9d8631b68e3adc57",
    "AWS::FSx::Snapshot": "656a8874092132a7",
    "AWS::FSx::StorageVirtualMachine": "b62638ce430b0942",
    "AWS::FSx::Volume": "e2217632738abb7f",
    "AWS::GameLift::Alias": "a0f3d4dec1e609ca",
    "AWS::GameLift::Build": "4158af836d4a87e4",
    "AWS::GameLift::Fleet": "3449469c24208290",
    "AWS::GlobalAccelerator::Accelerator": "7e13b677c20d567e",
    "AWS::GlobalAccelerator::CrossAccountAttachment": "e32f67162970b704",
    "AWS::GlobalAccelerator::EndpointGroup": "e6b735f07a95d38f",
    "AWS::GlobalAccelerator::Listener": "e4ddc6bb4a9202a8",
    "AWS::Glue::Classifier": "e64237d854ac7fec",
    "AWS::Glue::Connection": "c1ae0f653bf6925b",
    "AWS::Glue::Crawler": "4d80d0fa90e5fedd",
    "AWS::Glue::CustomEntityType": "b46c7cf573d72e3d",
    "AWS::Glue::DataCatalogEncryptionSettings": "e4d7c9e644714bf4",
    "AWS::Glue::DataQualityRuleset": "faf8cbb22cb87066",
    "AWS::Glue::Database": "3649b59bf6fd8105",
    "AWS::Glue::DevEndpoint": "e620bcbf039ad200",
    "AWS::Glue::Job": "e7c6608d4c02d1e4",
    "AWS::Glue::MLTransform": "2c5a75b004c29815",
    "AWS::Glue::Partition": "b1116e6b758cc144",
    "AWS::Glue::SecurityConfiguration": "ab65531ec60b84d0",
    "AWS::Glue::Table": "2528cdc058dacdf8",
    "AWS::Glue::Trigger": "d61fdd6249ab4ce9",
    "AWS::Glue::UsageProfile": "c43f6252ac5a4f73",
    "AWS::Glue::Workflow": "7889f86fa2b1958f",
    "AWS::GuardDuty::Detector": "609c5ab9768f30a4",
    "AWS::GuardDuty::Filter": "5effc026fb87131d",
    "AWS::GuardDuty::IPSet": "2755befa61203d7e",
    "AWS::GuardDuty::MalwareProtectionPlan": "d875ebc427de851e",
    "AWS::GuardDuty::Master": "5115377f51b894ca",
    "AWS::GuardDuty::Member": "4410bf4350c9fdb8",
    "AWS::GuardDuty::PublishingDestination": "c0e7c21001df88a4",
    "AWS::GuardDuty::ThreatEntitySet": "f9e73b098cba1e1d",
    "AWS::GuardDuty::ThreatIntelSet": "41eb1d839da4d715",
    "AWS::GuardDuty::TrustedEntitySet": "150ef5b439192da2",
    "AWS::IAM::AccessKey": "62d91b17309e76c8",
    "AWS::IAM::Group": "23f995735d09bdb9",
    "AWS::IAM::GroupPolicy": "afebc6bd2ca04f23",
    "AWS::IAM::InstanceProfile": "08ce3f200d1d387a",
    "AWS::IAM::ManagedPolicy": "1fab4f98707c4a5c",
    "AWS::IAM::OIDCProvider": "fffa104653a5e49e",
    "AWS::IAM::Policy": "a09a4b402c91ab00",
    "AWS::IAM::Role": "7fefd2b7982c788c",
    "AWS::IAM::RolePolicy": "cf758bdbc12fe737",
    "AWS::IAM::SAMLProvider": "3d4001db72ea8aee",
    "AWS::IAM::ServerCertificate": "5840107f711b833c",
    "AWS::IAM::ServiceLinkedRole": "9835b29e268f3901",
    "AWS::IAM::User": "fa88f766d15e3dd7",
    "AWS::IAM::UserPolicy": "dc178cbb13b36149",
    "AWS::IAM::UserToGroupAddition": "913f67f68af9fd37",
    "AWS::IdentityStore::Group": "1b0e7a9a2bec57b5",
    "AWS::IdentityStore::GroupMembership": "d010a01463729806",
    "AWS::ImageBuilder::Component": "95f58a3a4a138e5e",
    "AWS::ImageBuilder::ContainerRecipe": "9ef7ae876461fbce",
    "AWS::ImageBuilder::DistributionConfiguration": "7b34e1b0b9958d96",
    "AWS::ImageBuilder::Image": "fea7db3bf6f817cb",
    "AWS::ImageBuilder::ImagePipeline": "b439c448d65a8542",
    "AWS::ImageBuilder::ImageRecipe": "37e69d9a511bb367",
    "AWS::ImageBuilder::InfrastructureConfiguration": "703bdd9fb961cbcd",
    "AWS::ImageBuilder::LifecyclePolicy": "05e1176be456713a",
    "AWS::ImageBuilder::Workflow": "13db705a33e231c9",
    "AWS::InspectorV2::CisScanConfiguration": "6a0dfa78d5d181cb",
    "AWS::InspectorV2::Filter": "075d0c9233498945",
    "AWS::InternetMonitor::Monitor": "acce608acc67f3a1",
    "AWS::IoT::Certificate": "8cd2a6a60e246ed4",
    "AWS::IoT::Policy": "fb64fbcfb68776b4",
    "AWS::IoT::PolicyPrincipalAttachment": "0e3884a378d7593b",
    "AWS::IoT::Thing": "ad049bc7cdd09406",
    "AWS::IoT::ThingPrincipalAttachment": "cb2db377e7001773",
    "AWS::IoT::TopicRule": "841d252188c3102b",
    "AWS::KMS::Alias": "28f10f922a4d194f",
    "AWS::KMS::Key": "c70e396ee12ddb65",
    "AWS::KMS::ReplicaKey": "903f43e74c8d8853",
    "AWS::KafkaConnect::Connector": "02c89e2c1eb3d4f3",
    "AWS::KafkaConnect::CustomPlugin": "b57a0879af8759b3",
    "AWS::KafkaConnect::WorkerConfiguration": "1f80a6b9f1a4d5be",
    "AWS::Kinesis::ResourcePolicy": "39996915810e234a",
    "AWS::Kinesis::Stream": "8ebde19042fda1b6",
    "AWS::Kinesis::StreamConsumer": "7ce125c877a406e0",
    "AWS::KinesisAnalyticsV2::Application": "09427c8594d9a4e2",
    "AWS::KinesisFirehose::DeliveryStream": "9c8f3215b7dfb3ab",
    "AWS::LakeFormation::DataCellsFilter": "6ac217ac96918929",
    "AWS::LakeFormation::DataLakeSettings": "82b950110e995bdf",
    "AWS::LakeFormation::Permissions": "062fc8932c9970c4",
    "AWS::LakeFormation::PrincipalPermissions": "89fd48f71918cde3",
    "AWS::LakeFormation::Resource": "64382446e2262067",
    "AWS::LakeFormation::Tag": "e6965562c3f7e5bb",
    "AWS::LakeFormation::TagAssociation": "22d7f7ee98ca2b17",
    "AWS::Lambda::Alias": "ffc716c47265c9f0",
    "AWS::Lambda::EventInvokeConfig": "eb90a4c91675fe09",
    "AWS::Lambda::EventSourceMapping": "6356d681c78e3605",
    "AWS::Lambda::Function": "733365f24febb27e",
    "AWS::Lambda::LayerVersion": "f00fbc621c084236",
    "AWS::Lambda::LayerVersionPermission": "3d77aa7f6f48b060",
    "AWS::Lambda::Permission": "a43a5328bea4f6ad",
    "AWS::Lambda::Version": "e7fcd54deb138582",
    "AWS::LaunchWizard::Deployment": "2532c1afdd9c1f65",
    "AWS::Logs::AccountPolicy": "02fea54f4157dda9",
    "AWS::Logs::Delivery": "c87395bbed1a6b98",
    "AWS::Logs::DeliveryDestination": "30b61a3515971eda",
    "AWS::Logs::DeliverySource": "67b64320b397d6cd",
    "AWS::Logs::Destination": "ff1e6b020f2cf8e7",
    "AWS::Logs::LogAnomalyDetector": "81724eda2e87fcfc",
    "AWS::Logs::LogGroup": "2678f72332d506c4",
    "AWS::Logs::LogStream": "91035e555c3110ac",
    "AWS::Logs::MetricFilter": "4b4400caff5cc44e",
    "AWS::Logs::QueryDefinition": "7b82b12db189c1d9",
    "AWS::Logs::ResourcePolicy": "d7898253a7730ac3",
    "AWS::Logs::SubscriptionFilter": "c1ec900772212dcf",
    "AWS::Logs::Transformer": "943d9e7f35c1fc2c",
    "AWS::MSK::BatchScramSecret": "ff2b807a577495b0",
    "AWS::MSK::Cluster": "07d7025a3ae5e3bb",
    "AWS::MSK::ClusterPolicy": "b60740da22ea8c04",
    "AWS::MSK::Configuration": "61dbb22c2b6cbbb1",
    "AWS::MSK::Replicator": "e4c147fed22ae544",
    "AWS::MSK::Topic": "7c3b1270fbb7dbf8",
    "AWS::MSK::VpcConnection": "6aeb07916b90c80a",
    "AWS::MWAA::Environment": "c28ab3624f1d638c",
    "AWS::MediaConnect::Flow": "88870702e634ac9b",
    "AWS::MediaConnect::FlowEntitlement": "b3c4a7557601ea28",
    "AWS::MediaConnect::FlowOutput": "f7e6788d9fae4b17",
    "AWS::MediaConnect::FlowSource": "0f185886ed9279cb",
    "AWS::MediaConnect::FlowVpcInterface": "9bf072888370d7b0",
    "AWS::MediaLive::CloudWatchAlarmTemplate": "81fb2ce45d067a00",
    "AWS::MediaLive::CloudWatchAlarmTemplateGroup": "6ce0be92d99cb079",
    "AWS::MediaLive::EventBridgeRuleTemplate": "546f92cd028358f3",
    "AWS::MediaLive::EventBridgeRuleTemplateGroup": "d600d2e2b26dcff9",
    "AWS::MediaLive::Multiplex": "01d8204ac37d509d",
    "AWS::MediaLive::Multiplexprogram": "158d6f9e8427ca85",
    "AWS::MediaLive::SignalMap": "4469cefa6a45f1f7",
    "AWS::MediaPackage::Asset": "46a05885cb8e8a23",
    "AWS::MediaPackage::PackagingConfiguration": "381cdf03053e7a94",
    "AWS::MediaPackage::PackagingGroup": "ef7cc14fb78ab5d0",
    "AWS::MediaPackageV2::Channel": "62a1a362e9e8f627",
    "AWS::MediaPackageV2::ChannelGroup": "2aa9c5f3fad67e76",
    "AWS::MediaPackageV2::ChannelPolicy": "e79aba75cf23e354",
    "AWS::MediaPackageV2::OriginEndpoint": "a0a1cea117c52357",
    "AWS::MediaPackageV2::OriginEndpointPolicy": "8cd12a52dceffffe",
    "AWS::MediaTailor::Channel": "fe2dcc2566975851",
    "AWS::MediaTailor::ChannelPolicy": "1c9cf783f5b18afb",
    "AWS::MediaTailor::LiveSource": "a884632fe226b938",
    "AWS::MediaTailor::PlaybackConfiguration": "45549228850e8811",
    "AWS::MediaTailor::SourceLocation": "7ddd02a26d6de0a4",
    "AWS::MediaTailor::VodSource": "746fa4c2fdf8d70a",
    "AWS::Neptune::DBCluster": "8d9215c18d4689e5",
    "AWS::Neptune::DBClusterParameterGroup": "d3f072d3d16a7a5f",
    "AWS::Neptune::DBInstance": "40928fad97bbb2cd",
    "AWS::Neptune::DBParameterGroup": "6b2f9cd1b2702ccf",
    "AWS::Neptune::DBSubnetGroup": "c59d053f84ba853c",
    "AWS::Neptune::EventSubscription": "291cb01f3cbffb3e",
    "AWS::NetworkFirewall::Firewall": "5734de8d83f8ab9a",
    "AWS::NetworkFirewall::FirewallPolicy": "23d185da5cfdceca",
    "AWS::NetworkFirewall::LoggingConfiguration": "24065655a3066a83",
    "AWS::NetworkFirewall::RuleGroup": "0a775a8261b19aad",
    "AWS::NetworkFirewall::TLSInspectionConfiguration": "c67eab10509c4b9d",
    "AWS::NetworkFirewall::VpcEndpointAssociation": "671a5fc51bc59b14",
    "AWS::NetworkManager::ConnectAttachment": "c60037d3397a6cc1",
    "AWS::NetworkManager::ConnectPeer": "46040e0b03a66bd1",
    "AWS::NetworkManager::CoreNetwork": "7a12f9db536d923a",
    "AWS::NetworkManager::CoreNetworkPrefixListAssociation": "8fe12d1955653312",
    "AWS::NetworkManager::CustomerGatewayAssociation": "61803ddea55493f6",
    "AWS::NetworkManager::Device": "b5f810a71f944539",
    "AWS::NetworkManager::DirectConnectGatewayAttachment": "12c6241616a15feb",
    "AWS::NetworkManager::GlobalNetwork": "ca5f62c7f8207621",
    "AWS::NetworkManager::Link": "f60526a20a0904b2",
    "AWS::NetworkManager::LinkAssociation": "c78f6eaf3131bc07",
    "AWS::NetworkManager::Site": "dee802b425d588b0",
    "AWS::NetworkManager::SiteToSiteVpnAttachment": "6ef3ab5fb2ddab36",
    "AWS::NetworkManager::TransitGatewayPeering": "6162001e30d64860",
    "AWS::NetworkManager::TransitGatewayRegistration": "5285e1ce76dd44a0",
    "AWS::NetworkManager::TransitGatewayRouteTableAttachment": "e08752976a5d6f68",
    "AWS::NetworkManager::VpcAttachment": "a06da0d509ded081",
    "AWS::ODB::CloudAutonomousVmCluster": "237f5696f1f714a4",
    "AWS::ODB::CloudExadataInfrastructure": "36dcb7fdacb13249",
    "AWS::ODB::CloudVmCluster": "321f0612cafea30e",
    "AWS::ODB::OdbNetwork": "cea1c28172eb56d8",
    "AWS::ODB::OdbPeeringConnection": "6f4a7881e1c35960",
    "AWS::Oam::Link": "4d4e114e0f615d82",
    "AWS::Oam::Sink": "81957ea027e4c201",
    "AWS::ObservabilityAdmin::OrganizationTelemetryRule": "b1afe16ecb07efd3",
    "AWS::ObservabilityAdmin::S3TableIntegration": "9dcd4f79cf41f229",
    "AWS::ObservabilityAdmin::TelemetryPipelines": "1acc5affc6615ad2",
    "AWS::ObservabilityAdmin::TelemetryRule": "4ea5670f96981494",
    "AWS::OpenSearchServerless::AccessPolicy": "ec7a348a493d8c4f",
    "AWS::OpenSearchServerless::Collection": "300399ba940599d2",
    "AWS::OpenSearchServerless::CollectionGroup": "b6d93309ca936d98",
    "AWS::OpenSearchServerless::Index": "fcc40949c8a1e83a",
    "AWS::OpenSearchServerless::LifecyclePolicy": "71de0a66a6f0daa7",
    "AWS::OpenSearchServerless::SecurityConfig": "4d3e887cb7677f9c",
    "AWS::OpenSearchServerless::SecurityPolicy": "a73437e0f606fd57",
    "AWS::OpenSearchServerless::VpcEndpoint": "e4bcf34472c7a694",
    "AWS::OpenSearchService::Application": "54acd8b539fb7b22",
    "AWS::OpenSearchService::Domain": "e7e6d71f19ceff49",
    "AWS::OpsWorks::App": "b03e7809117d5e3e",
    "AWS::OpsWorks::ElasticLoadBalancerAttachment": "b255cbf1e79baaa4",
    "AWS::OpsWorks::Instance": "9eef779da07ead94",
    "AWS::OpsWorks::Layer": "cc10c4f050be03ae",
    "AWS::OpsWorks::Stack": "8797f81c822cbdd8",
    "AWS::OpsWorks::UserProfile": "02aeb7e785d996d6",
    "AWS::OpsWorks::Volume": "1596706b00045a5a",
    "AWS::Organizations::Account": "5e4fa45032c79057",
    "AWS::Organizations::Organization": "ef5f3cbaa91637bd",
    "AWS::Organizations::OrganizationalUnit": "48f9a21ef5f81812",
    "AWS::Organizations::Policy": "91a30b5718ac046b",
    "AWS::Organizations::ResourcePolicy": "c155f5ddea5b4398",
    "AWS::PCAConnectorAD::Connector": "956000172e4b20cd",
    "AWS::PCAConnectorAD::DirectoryRegistration": "1dc615cca69549dd",
    "AWS::PCAConnectorAD::ServicePrincipalName": "6a8ea7b9a179bc85",
    "AWS::PCAConnectorAD::Template": "ed0b568c05022bdf",
    "AWS::PCAConnectorAD::TemplateGroupAccessControlEntry": "756784fc38981208",
    "AWS::PCAConnectorSCEP::Challenge": "66f2129c2055583e",
    "AWS::PCAConnectorSCEP::Connector": "3985d16f9d690d78",
    "AWS::PaymentCryptography::Alias": "04360ee15415de2d",
    "AWS::PaymentCryptography::Key": "af26a71fe21eb480",
    "AWS::Pipes::Pipe": "ef04c5b621766de2",
    "AWS::RAM::Permission": "978c532747f94c7b",
    "AWS::RAM::ResourceShare": "b4b5ff31bc5f95b0",
    "AWS::RDS::DBCluster": "089832d6c2149c95",
    "AWS::RDS::DBClusterParameterGroup": "4e3460620664c07f",
    "AWS::RDS::DBInstance": "25726c9e8467f4c5",
    "AWS::RDS::DBParameterGroup": "d29cf38b861f6048",
    "AWS::RDS::DBProxy": "308e70305b33a892",
    "AWS::RDS::DBProxyEndpoint": "17943cc672d084c2",
    "AWS::RDS::DBProxyTargetGroup": "b584b6fb42fcd4c7",
    "AWS::RDS::DBSecurityGroup": "42f555e782cd9946",
    "AWS::RDS::DBSecurityGroupIngress": "1a14ad6e73c0b6b7",
    "AWS::RDS::DBShardGroup": "df6b36c7e099595a",
    "AWS::RDS::DBSubnetGroup": "4a2ab41cbcc2adb8",
    "AWS::RDS::EventSubscription": "85bf3af749d312f0",
    "AWS::RDS::GlobalCluster": "a87d030f06c19d47",
    "AWS::RDS::Integration": "b41a67e5adeb023f",
    "AWS::RDS::OptionGroup": "d89f528bd7bdd696",
    "AWS::RUM::AppMonitor": "02162c9e29b60596",
    "AWS::Rbin::Rule": "7302f3e4816f5e9e",
    "AWS::Redshift::Cluster": "24380b75d618dc41",
    "AWS::Redshift::ClusterParameterGroup": "5eafb174c8375f06",
    "AWS::Redshift::ClusterSecurityGroup": "6a629309a94b6faa",
    "AWS::Redshift::ClusterSecurityGroupIngress": "0079cffa4eeb3685",
    "AWS::Redshift::ClusterSubnetGroup": "5c5ef039a548a325",
    "AWS::Redshift::EndpointAccess": "c8b246db37a986a0",
    "AWS::Redshift::EndpointAuthorization": "e72be424d109da1d",
    "AWS::Redshift::EventSubscription": "636e5903e3e73299",
    "AWS::Redshift::Integration": "ea8ecb64062e3355",
    "AWS::Redshift::ScheduledAction": "8f8f379cd2fdb2bc",
    "AWS::RedshiftServerless::Namespace": "5713f5387741d15b",
    "AWS::RedshiftServerless::Snapshot": "826c5d08e248c956",
    "AWS::RedshiftServerless::Workgroup": "20492b7e2170a5a9",
    "AWS::ResourceExplorer2::DefaultViewAssociation": "a7cd7349891d3ea3",
    "AWS::ResourceExplorer2::Index": "b67cbdd9ae9d9f39",
    "AWS::ResourceExplorer2::View": "f8b36a72ee1a6bd3",
    "AWS::ResourceGroups::Group": "7be20625159a2551",
    "AWS::ResourceGroups::TagSyncTask": "9683ef6d409e4585",
    "AWS::RolesAnywhere::CRL": "f52a01aa06e590f5",
    "AWS::RolesAnywhere::Profile": "e6cb05cf67e91208",
    "AWS::RolesAnywhere::TrustAnchor": "3add0281e203381f",
    "AWS::Route53::CidrCollection": "2332f11a72446133",
    "AWS::Route53::DNSSEC": "106ee68fc4cc0a3c",
    "AWS::Route53::HealthCheck": "92f14427097812fb",
    "AWS::Route53::HostedZone": "ff22e0187d9beac4",
    "AWS::Route53::KeySigningKey": "a9a67ae6c371fcf4",
    "AWS::Route53::RecordSet": "c01e9eccec2e3b83",
    "AWS::Route53::RecordSetGroup": "0f70c9c554987b2d",
    "AWS::Route53GlobalResolver::AccessSource": "5b44826009d95a82",
    "AWS::Route53GlobalResolver::AccessToken": "ebce066c0deb24b4",
    "AWS::Route53GlobalResolver::DnsView": "b1b7b3c050f0372a",
    "AWS::Route53GlobalResolver::FirewallDomainList": "ec856b8678fa09a4",
    "AWS::Route53GlobalResolver::FirewallRule": "cb1f3a4572521668",
    "AWS::Route53GlobalResolver::GlobalResolver": "d000d28ecc222db8",
    "AWS::Route53GlobalResolver::HostedZoneAssociation": "19d1204761dc53db",
    "AWS::Route53Profiles::Profile": "1d89ef4356c426b4",
    "AWS::Route53Profiles::ProfileAssociation": "fa568e44f7d9e9f2",
    "AWS::Route53Profiles::ProfileResourceAssociation": "73d6b9da50afa75b",
    "AWS::Route53Resolver::FirewallDomainList": "47cc5f401e1f9bbe",
    "AWS::Route53Resolver::FirewallRuleGroup": "bee6df8eb043a885",
    "AWS::Route53Resolver::FirewallRuleGroupAssociation": "d84c68a6ebbb5876",
    "AWS::Route53Resolver::ResolverConfig": "174be26547caaa42",
    "AWS::Route53Resolver::ResolverDNSSECConfig": "813fd8fe1e24c6e3",
    "AWS::Route53Resolver::ResolverEndpoint": "fc74be0ae84138f0",
    "AWS::Route53Resolver::ResolverQueryLoggingConfig": "896173cca4e29ac8",
    "AWS::Route53Resolver::ResolverQueryLoggingConfigAssociation": "41e93f940f8bce93",
    "AWS::Route53Resolver::ResolverRule": "455bd6c59c07fb7b",
    "AWS::Route53Resolver::ResolverRuleAssociation": "3e5cdfc8643c083b",
    "AWS::S3::AccessGrant": "b1e7af0495a078b0",
    "AWS::S3::AccessGrantsInstance": "05d3e874b43bbad8",
    "AWS::S3::AccessGrantsLocation": "3a7be1727f630029",
    "AWS::S3::AccessPoint": "e8b95810dd8d6917",
    "AWS::S3::Bucket": "5722eb3563dbd62b",
    "AWS::S3::BucketPolicy": "38ad5b8bd8062535",
    "AWS::S3::MultiRegionAccessPoint": "193323be1bf2dac0",
    "AWS::S3::MultiRegionAccessPointPolicy": "7ac84076052ddd16",
    "AWS::S3ObjectLambda::AccessPoint": "2d6b2b82a9508114",
    "AWS::S3ObjectLambda::AccessPointPolicy": "fd6328c6c782223a",
    "AWS::S3Tables::Namespace": "148f9286df26a384",
    "AWS::S3Tables::Table": "06c0bb9325f0bc8c",
    "AWS::S3Tables::TableBucket": "c87800af01ff17ff",
    "AWS::S3Tables::TableBucketPolicy": "8a4df56718b22c4c",
    "AWS::S3Tables::TablePolicy": "a400a63607882bc8",
    "AWS::SDB::Domain": "48c38397df169b43",
    "AWS::SES::ConfigurationSet": "5d4781bc5cccba7b",
    "AWS::SES::ConfigurationSetEventDestination": "f4e74b1d7e859c34",
    "AWS::SES::ContactList": "c5a40d0f3d530e19",
    "AWS::SES::CustomVerificationEmailTemplate": "4f52e41f83844ece",
    "AWS::SES::DedicatedIpPool": "1485a5989d771f62",
    "AWS::SES::EmailIdentity": "be6211158f395729",
    "AWS::SES::MailManagerAddonInstance": "2012772f13a846eb",
    "AWS::SES::MailManagerAddonSubscription": "63e6d6434e3ac413",
    "AWS::SES::MailManagerAddressList": "22302ded05b5fc2a",
    "AWS::SES::MailManagerArchive": "3f33022e6c705d6d",
    "AWS::SES::MailManagerIngressPoint": "923b3c23a9a15b1b",
    "AWS::SES::MailManagerRelay": "92102c7d273b1d47",
    "AWS::SES::MailManagerRuleSet": "8fd084ee266114ee",
    "AWS::SES::MailManagerTrafficPolicy": "e229aa555378d193",
    "AWS::SES::MultiRegionEndpoint": "23f0206212864cb3",
    "AWS::SES::Template": "b50788c843c91468",
    "AWS::SES::Tenant": "0c003f20faaf4446",
    "AWS::SES::VdmAttributes": "15df0379061c0489",
    "AWS::SMSVOICE::ConfigurationSet": "2bdda88e956e2a26",
    "AWS::SMSVOICE::OptOutList": "005f2438fbf815f8",
    "AWS::SMSVOICE::PhoneNumber": "a70df49ea7eda8ef",
    "AWS::SMSVOICE::Pool": "6b463c5c9b1cba38",
    "AWS::SMSVOICE::ProtectConfiguration": "2323c25bc333ab42",
    "AWS::SMSVOICE::ResourcePolicy": "1e13b6a86622c5ce",
    "AWS::SMSVOICE::SenderId": "28ee6b42794a5db3",
    "AWS::SNS::Subscription": "9426c559a25e9da6",
    "AWS::SNS::Topic": "210149c7ee544cd1",
    "AWS::SNS::TopicInlinePolicy": "e47e6b028a7c3cc7",
    "AWS::SNS::TopicPolicy": "04470a92c9a1130c",
    "AWS::SQS::Queue": "bc980a1f0e10a6fa",
    "AWS::SQS::QueueInlinePolicy": "e4132d6c68dcbc6c",
    "AWS::SQS::QueuePolicy": "8d0c3b8154e71ff3",
    "AWS::SSM::Association": "e5ce83b562338270",
    "AWS::SSM::Document": "fafd81d675e3af92",
    "AWS::SSM::MaintenanceWindow": "8bcf8299c61efe9b",
    "AWS::SSM::MaintenanceWindowTarget": "1d59aec599d718b8",
    "AWS::SSM::MaintenanceWindowTask": "dbf020e1bce02a1c",
    "AWS::SSM::Parameter": "88998b8e50aa846a",
    "AWS::SSM::PatchBaseline": "43cce065b6753223",
    "AWS::SSM::ResourceDataSync": "f92ce1aa6674b7f2",
    "AWS::SSO::Application": "55ac5b9ce8741467",
    "AWS::SSO::ApplicationAssignment": "371bae918d5c9ea1",
    "AWS::SSO::Assignment": "3e0b05f0a3cba8d8",
    "AWS::SSO::Instance": "dd7d415516c90a5e",
    "AWS::SSO::InstanceAccessControlAttributeConfiguration": "ccfedd9599046cb1",
    "AWS::SSO::PermissionSet": "e4212a96dd4b627d",
    "AWS::SageMaker::MlflowTrackingServer": "82716f2317f73364",
    "AWS::SageMaker::ModelCard": "89534938581c1015",
    "AWS::SageMaker::Pipeline": "d0698ac81fb18f57",
    "AWS::SageMaker::ProcessingJob": "e1994976600f5bae",
    "AWS::Scheduler::Schedule": "e8b60514af25a837",
    "AWS::Scheduler::ScheduleGroup": "ef710d09dbb243bf",
    "AWS::SecretsManager::ResourcePolicy": "cb5d330bf15d9d5a",
    "AWS::SecretsManager::RotationSchedule": "e02692fb717f1c08",
    "AWS::SecretsManager::Secret": "ea029b329dc57364",
    "AWS::SecretsManager::SecretTargetAttachment": "3bf4404734c82308",
    "AWS::SecurityHub::AutomationRule": "f42a9d0542caf822",
    "AWS::SecurityHub::AutomationRuleV2": "cd28c1b3f7e237bc",
    "AWS::SecurityHub::ConnectorV2": "b4e780d54c354d65",
    "AWS::SecurityHub::DelegatedAdmin": "a328fc62a9f2f7e2",
    "AWS::SecurityHub::Hub": "9f6cb2d8967bfcf6",
    "AWS::SecurityHub::HubV2": "b1544234a77fc9a5",
    "AWS::SecurityHub::Insight": "6c43c37d361a91ef",
    "AWS::SecurityHub::OrganizationConfiguration": "494d5bdaaa99a7a9",
    "AWS::SecurityHub::ProductSubscription": "a6a7b09a786ba544",
    "AWS::SecurityHub::SecurityControl": "062ab72de4bbcd1d",
    "AWS::SecurityHub::Standard": "a9a57a7527f5aff5",
    "AWS::ServiceCatalog::AcceptedPortfolioShare": "77373a56c0c4bd28",
    "AWS::ServiceCatalog::CloudFormationProduct": "662186534ff8e643",
    "AWS::ServiceCatalog::CloudFormationProvisionedProduct": "448a377417beb2e5",
    "AWS::ServiceCatalog::LaunchNotificationConstraint": "b7978856af5cccf2",
    "AWS::ServiceCatalog::LaunchRoleConstraint": "e953e967aeda94a8",
    "AWS::ServiceCatalog::LaunchTemplateConstraint": "e457535596ed60a2",
    "AWS::ServiceCatalog::Portfolio": "4a1c306059a85c39",
    "AWS::ServiceCatalog::PortfolioPrincipalAssociation": "c1655e6cd9e94bc0",
    "AWS::ServiceCatalog::PortfolioProductAssociation": "39311ae1cb0b583e",
    "AWS::ServiceCatalog::PortfolioShare": "c365f2118140f92b",
    "AWS::ServiceCatalog::ResourceUpdateConstraint": "9ddbc905c8a71617",
    "AWS::ServiceCatalog::ServiceAction": "d7e4fd1cc71dc961",
    "AWS::ServiceCatalog::ServiceActionAssociation": "283d0ac6cbcfde8a",
    "AWS::ServiceCatalog::StackSetConstraint": "8e87a7069c86c892",
    "AWS::ServiceCatalog::TagOption": "0a7e7604f593ea6c",
    "AWS::ServiceCatalog::TagOptionAssociation": "932ff2b84208a648",
    "AWS::ServiceCatalogAppRegistry::Application": "b2e53112fb9f6f30",
    "AWS::ServiceCatalogAppRegistry::AttributeGroup": "8467fd4f3c6134b3",
    "AWS::ServiceCatalogAppRegistry::AttributeGroupAssociation": "5593933d7ee4cd44",
    "AWS::ServiceCatalogAppRegistry::ResourceAssociation": "948d7080333f163c",
    "AWS::ServiceDiscovery::HttpNamespace": "986cb51cac464fe8",
    "AWS::ServiceDiscovery::Instance": "b4227e1dfa14a394",
    "AWS::ServiceDiscovery::PrivateDnsNamespace": "d78a2213969ef282",
    "AWS::ServiceDiscovery::PublicDnsNamespace": "da152fa1f7dce966",
    "AWS::ServiceDiscovery::Service": "cc0f82ca0c9cf09a",
    "AWS::Shield::DRTAccess": "fb21794f4659ef00",
    "AWS::Shield::ProactiveEngagement": "8eb499fd79e0fac5",
    "AWS::Shield::Protection": "4d78aae6b8d58b77",
    "AWS::Shield::ProtectionGroup": "56265d6029ba5add",
    "AWS::StepFunctions::Activity": "b3156a89a45815e1",
    "AWS::StepFunctions::StateMachine": "f6e6ebffca5675b3",
    "AWS::StepFunctions::StateMachineAlias": "55cc66ee1f33c9f5",
    "AWS::StepFunctions::StateMachineVersion": "4467293a52255b78",
    "AWS::Synthetics::Canary": "a553d010d467c6ed",
    "AWS::SystemsManagerSAP::Application": "8196ab023743f75e",
    "AWS::Transfer::Agreement": "1013512feba974f8",
    "AWS::Transfer::Certificate": "f7478973edec7bd3",
    "AWS::Transfer::Connector": "8827021611524252",
    "AWS::Transfer::Profile": "88c7478695f7c070",
    "AWS::Transfer::Server": "90b633961fce2a5f",
    "AWS::Transfer::User": "7dbec614b03c1346",
    "AWS::Transfer::WebApp": "146273f9ed9978d6",
    "AWS::Transfer::Workflow": "851674d5732910da",
    "AWS::VerifiedPermissions::IdentitySource": "b6f4f19263afee23",
    "AWS::VerifiedPermissions::Policy": "f7e75c34791cf6a5",
    "AWS::VerifiedPermissions::PolicyStore": "cd1f710f1ce7122f",
    "AWS::VerifiedPermissions::PolicyTemplate": "067128efc0e8f2eb",
    "AWS::VpcLattice::AccessLogSubscription": "665daf045fc98c65",
    "AWS::VpcLattice::AuthPolicy": "4265f5022282b0dc",
    "AWS::VpcLattice::DomainVerification": "e7469383aed1b9e4",
    "AWS::VpcLattice::Listener": "060bfe3c19acb621",
    "AWS::VpcLattice::ResourceConfiguration": "44271e8bd9b0bfb2",
    "AWS::VpcLattice::ResourceGateway": "3894adacfadd29c3",
    "AWS::VpcLattice::ResourcePolicy": "abb0517e24f452b5",
    "AWS::VpcLattice::Rule": "a99a25f3ef10925a",
    "AWS::VpcLattice::Service": "aca343366a1dc0f5",
    "AWS::VpcLattice::ServiceNetwork": "d15e63a95b8b2180",
    "AWS::VpcLattice::ServiceNetworkResourceAssociation": "15b389795ecd77f8",
    "AWS::VpcLattice::ServiceNetworkServiceAssociation": "79f2001afdfc5c64",
    "AWS::VpcLattice::ServiceNetworkVpcAssociation": "f29a2d687d5fb5d0",
    "AWS::VpcLattice::TargetGroup": "82eedcc7b4d3cb1b",
    "AWS::WAF::ByteMatchSet": "f82cc1eb2b0fc3a3",
    "AWS::WAF::IPSet": "18b56ae5ec12e194",
    "AWS::WAF::Rule": "36e0ade317983947",
    "AWS::WAF::SizeConstraintSet": "0be73e32606a7a20",
    "AWS::WAF::SqlInjectionMatchSet": "5c0737e41a7a90b2",
    "AWS::WAF::WebACL": "83186fa64f0217b3",
    "AWS::WAF::XssMatchSet": "8c91ae009d1e5c97",
    "AWS::WAFv2::IPSet": "fe4bc7d488064ebc",
    "AWS::WAFv2::LoggingConfiguration": "219a7415589ec0bf",
    "AWS::WAFv2::RegexPatternSet": "71e5110f182eba06",
    "AWS::WAFv2::RuleGroup": "17c881f134c11dbb",
    "AWS::WAFv2::WebACL": "27eb9031e644a7fd",
    "AWS::WAFv2::WebACLAssociation": "f47dba384d872bb3",
    "AWS::WorkSpaces::Workspace": "95eec1aff41966b4",
    "AWS::XRay::Group": "9e6b73a50d7628e1",
    "AWS::XRay::ResourcePolicy": "1eefd10339461fe6",
    "AWS::XRay::SamplingRule": "fca746c2d6608b1a",
    "AWS::XRay::TransactionSearchConfig": "6bf850e284a67fbb",
    "Module": "baab0e57f109673a",
}



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules import CloudFormationLintRule


class ResourceLifecycleMaintenance(CloudFormationLintRule):
    id = "W3697"
    shortdesc = "Resource type is from a service in maintenance mode"
    description = (
        "Checks if a resource type belongs to an AWS service that is "
        "in maintenance mode with no new features"
    )
    source_url = (
        "https://docs.aws.amazon.com/general/latest/gr/maintenance_services.html"
    )
    tags = ["resources", "lifecycle"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.jsonschema.CfnLintKeyword import CfnLintKeyword


class FunctionEnvironmentSize(CfnLintKeyword):
    """Check Lambda environment variables total size does not exceed 4 KB"""

    id = "E3697"
    shortdesc = "Validate Lambda environment variables do not exceed 4 KB"
    description = (
        "AWS Lambda limits the total size of all environment variables "
        "to 4 KB. If this limit is exceeded, the deployment will fail. "
        "This rule sums the lengths of all keys and values and validates "
        "the total does not exceed 4096 bytes."
    )
    source_url = (
        "https://docs.aws.amazon.com/lambda/latest/dg/gettingstarted-limits.html"
    )
    tags = ["resources", "lambda"]

    def __init__(self):
        """Init"""
        super().__init__(
            ["Resources/AWS::Lambda::Function/Properties/Environment/Variables"]
        )

    def validate(
        self,
        validator: Validator,
        keywords: Any,
        instance: Any,
        schema: dict[str, Any],
    ) -> ValidationResult:
        if not validator.is_type(instance, "object"):
            return

        total_size = 0
        for key, value in instance.items():
            if not isinstance(key, str):
                continue
            total_size += len(key)
            if isinstance(value, str):
                total_size += len(value)

        max_size = 4096
        if total_size > max_size:
            yield ValidationError(
                f"Lambda environment variables total size ({total_size}) "
                f"exceeds the 4 KB ({max_size} bytes) limit",
                rule=self,
            )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_lambda_eventsourcemapping
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class EventSourceMappingSqsFifoBatchSize(CfnLintJsonSchema):
    id = "E3705"
    shortdesc = "Validate SQS FIFO queue EventSourceMapping BatchSize is at most 10"
    description = (
        "When an EventSourceMapping references a FIFO SQS queue, "
        "the BatchSize must be at most 10"
    )
    source_url = "https://docs.aws.amazon.com/lambda/latest/dg/with-sqs.html"
    tags = ["resources", "lambda", "sqs"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::Lambda::EventSourceMapping/Properties",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_lambda_eventsourcemapping,
                filename="event_source_sqs_fifo_batch_size.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any

from cfnlint.helpers import VALID_PARAMETER_TYPES
from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.jsonschema import CfnLintKeyword


class UnsupportedParameterType(CfnLintKeyword):
    """Check if Parameters use unsupported types"""

    id = "W2002"
    shortdesc = "Parameter type is not officially supported by CloudFormation"
    description = (
        "CloudFormation accepts any AWS::SSM::Parameter::Value<> or List<> pattern, "
        "but only validates specific types. Using unsupported types may work but "
        "CloudFormation will not validate the parameter values."
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cloudformation-supplied-parameter-types.html"
    tags = ["parameters"]

    def __init__(self) -> None:
        super().__init__(keywords=["Parameters/*/Type"])

    def validate(
        self, validator: Validator, s: Any, instance: Any, schema: Any
    ) -> ValidationResult:
        if not validator.is_type(instance, "string"):
            return

        # Check if it's an SSM or List type but not in the supported list
        if (
            instance.startswith("AWS::SSM::Parameter::Value<")
            or instance.startswith("List<")
        ) and instance not in VALID_PARAMETER_TYPES:
            yield ValidationError(
                f"{instance!r} is not an officially documented "
                "CloudFormation parameter type. While CloudFormation may "
                "accept this type, it will not validate the parameter value.",
                rule=self,
            )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules.resources.iam.Policy import Policy


class TrustPolicy(Policy):
    """Check IAM trust Policies"""

    id = "E3530"
    shortdesc = "Validate IAM trust polices"
    description = (
        "IAM trust polices are embedded JSON in CloudFormation. "
        "This rule validates those embedded policies."
    )
    source_url = "https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_principal.html"
    tags = ["resources", "iam"]

    def __init__(self):
        super().__init__(
            [
                "Resources/AWS::IAM::Role/Properties/AssumeRolePolicyDocument",
            ],
            "trust",
            "policy_trust.json",
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_ecs_service
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class ServiceFargate(CfnLintJsonSchema):
    id = "E3054"
    shortdesc = (
        "Validate ECS service using Fargate uses TaskDefinition that allows Fargate"
    )
    description = (
        "When using an ECS service with 'LaunchType' of 'FARGATE' "
        "the associated task definition must have 'RequiresCompatibilities' "
        "specified with 'FARGATE' listed"
    )
    tags = ["resources", "ecs"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::ECS::Service/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_ecs_service,
                filename="service_fargate.json",
            ),
            all_matches=True,
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        if err.validator == "required":
            return "'RequiresCompatibilities' is a required property"
        if err.validator == "contains":
            value = err.instance
            return f"{value!r} does not contain items matching 'FARGATE'"
        return err.message

    def _clean_error(self, err: ValidationError):
        err = super()._clean_error(err)
        if err.rule == self:
            err.message = self.message(None, err)
        return err



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules import CloudFormationLintRule


class ResourceLifecycleSunset(CloudFormationLintRule):
    id = "W3696"
    shortdesc = "Resource type is from a service that is sunsetting"
    description = (
        "Checks if a resource type belongs to an AWS service that is "
        "in the sunset phase and will be shut down"
    )
    source_url = "https://docs.aws.amazon.com/general/latest/gr/sunset_services.html"
    tags = ["resources", "lifecycle"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any

from cfnlint.jsonschema import ValidationError, Validator
from cfnlint.rules import CloudFormationLintRule


class DynamicReferenceSpaces(CloudFormationLintRule):
    id = "W1053"
    shortdesc = "Dynamic references should not contain spaces"
    description = (
        "Dynamic references with spaces between '{{' and 'resolve' "
        "will not be resolved by CloudFormation and will be treated as "
        "a literal string"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/dynamic-references.html"
    tags = ["functions", "dynamic reference"]

    def dynamicReferenceSpaces(
        self, validator: Validator, s: Any, instance: Any, schema: Any
    ):
        yield ValidationError(
            f"{instance!r} has spaces and will not be resolved as a "
            f"dynamic reference. Remove spaces from '{{{{resolve:...}}}}'",
            rule=self,
        )



"""
Copyright (c) 2013 Julian Berman

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

SPDX-License-Identifier: MIT
"""

# Code is taken from jsonschema package and adapted CloudFormation use
# https://github.com/python-jsonschema/jsonschema
from __future__ import annotations

from collections import deque
from typing import Any

import regex as re

import cfnlint.jsonschema._keywords as validators_standard
from cfnlint.helpers import (
    BOOLEAN_STRINGS,
    FUNCTION_FOR_EACH,
    FUNCTION_TRANSFORM,
    ensure_list,
    is_function,
)
from cfnlint.jsonschema import ValidationError, Validator
from cfnlint.jsonschema._typing import V, ValidationResult
from cfnlint.rules.helpers.get_value_from_path import get_value_from_path

_ap_exception_fns = set([FUNCTION_TRANSFORM, FUNCTION_FOR_EACH])

_UNRESOLVED = object()


def resolve_data_reference(value: Any, root_instance: Any) -> Any:
    """
    Resolve $data and $lookup references in schema values.

    $data uses absolute JSON Pointers to reference values in the instance.
    Example: {"const": {"$data": "/source/fifo"}}

    $lookup resolves a key via $data then maps it through a lookup table.
    Example: {"const": {"$lookup": {"key": {"$data": "/target/type"},
    "map": {"AWS::S3::Bucket": "s3.amazonaws.com"}}}}
    """
    if not isinstance(value, dict):
        return value

    if "$lookup" in value:
        lookup = value["$lookup"]
        if not isinstance(lookup, dict):
            return value
        key = resolve_data_reference(lookup.get("key"), root_instance)
        lookup_map = lookup.get("map", {})
        if isinstance(key, str) and key in lookup_map:
            return lookup_map[key]
        return _UNRESOLVED

    if "$data" not in value:
        return value

    pointer = value["$data"]
    if not isinstance(pointer, str) or not pointer.startswith("/"):
        return value

    # Navigate the JSON pointer
    parts = pointer.strip("/").split("/")
    current = root_instance

    try:
        for part in parts:
            if isinstance(current, dict):
                current = current[part]
            elif isinstance(current, list):
                current = current[int(part)]
            else:
                return value  # Can't navigate further
        return current
    except (KeyError, IndexError, ValueError):
        return value  # Reference not found, return original


def resolve_data_in_schema(schema: Any, root_instance: Any) -> Any:
    """
    Recursively resolve all $data references in a schema.
    """
    _numeric_keywords = {
        "minimum",
        "maximum",
        "exclusiveMinimum",
        "exclusiveMaximum",
    }
    _integer_keywords = {
        "minLength",
        "maxLength",
        "minItems",
        "maxItems",
    }
    _data_keywords = (
        _numeric_keywords
        | _integer_keywords
        | {
            "const",
            "enum",
            "pattern",
        }
    )
    if isinstance(schema, dict):
        result = {}
        for k, v in schema.items():
            if k in _data_keywords:
                resolved = resolve_data_reference(v, root_instance)
                if resolved is _UNRESOLVED:
                    continue
                if k in _numeric_keywords:
                    try:
                        f = float(resolved)
                        resolved = int(f) if f == int(f) else f
                    except (ValueError, TypeError):
                        continue
                elif k in _integer_keywords:
                    try:
                        resolved = int(resolved)
                    except (ValueError, TypeError):
                        continue
                result[k] = resolved
            else:
                result[k] = resolve_data_in_schema(v, root_instance)
        return result
    elif isinstance(schema, list):
        return [resolve_data_in_schema(item, root_instance) for item in schema]
    else:
        return schema


def additionalProperties(
    validator: Validator, aP: Any, instance: Any, schema: Any
) -> ValidationResult:
    # is function will just return if one item is present
    # as this is the standard. We will handle exceptions below
    k, _ = is_function(instance)
    if k in validator.context.functions:
        return
    for err in validators_standard.additionalProperties(
        validator, aP, instance, schema
    ):
        # Some functions can exist at the same level
        # so we need to validate that if those functions are
        # currently supported by the context and are part of the
        # error

        # if the path is 0 just yield the error and return
        # this should never happen
        if not len(err.path) > 0:  # pragma: no cover
            yield err  # pragma: no cover
            return  # pragma: no cover

        for fn in list(_ap_exception_fns & set(validator.context.functions)):
            if re.fullmatch(fn, str(err.path[0])):  # type: ignore
                break
        else:
            yield err


def cfnContext(
    validator: Validator,
    s: Any,
    instance: Any,
    schema: Any,
) -> ValidationResult:
    context_parameters: dict[str, Any] = {}

    functions = s.get("functions")
    if functions is not None:
        if validator.is_type(functions, "object"):
            if "$ref" in functions:
                _, functions = validator.resolver.resolve(functions["$ref"])
            else:
                functions = []

    pseudo_parameters = s.get("pseudoParameters")
    if pseudo_parameters is not None:
        context_parameters["pseudo_parameters"] = set(pseudo_parameters)

    references = s.get("references")
    if references is not None:
        if "Resources" not in references:
            context_parameters["resources"] = {}

    if functions is not None:
        context_parameters["functions"] = functions

    cfn_validator = validator.evolve(
        context=validator.context.evolve(**context_parameters)
    )

    yield from cfn_validator.descend(
        instance=instance,
        schema=s.get("schema", {}),
        schema_path="schema",
    )


def dynamicValidation(
    validator: Validator, dV: Any, instance: Any, schema: dict[str, Any]
) -> ValidationResult:
    """
    Performs dynamic validation based on context.

    The dynamicValidation keyword supports:
    - context: The context source to validate against
               (parameters, conditions, resources, etc.)
    - transformCheck: Check if a specific transform exists
                      (returns true/false for if/then/else)
    - pathCheck: Validate based on the current path in the template
    """
    if not validator.is_type(dV, "object"):
        return

    # Handle transform check (for use with if/then/else)
    transform_check = dV.get("transformCheck")
    if transform_check is not None:
        if transform_check not in validator.context.transforms.transforms:
            yield ValidationError(
                f"Transform {transform_check!r} is required "
                "but not present in the template"
            )

    # Handle dynamic source validation
    context_source = dV.get("context")
    if context_source is not None:
        if context_source and isinstance(context_source, str):
            # Get the appropriate collection based on the context source
            collection = None
            if context_source == "conditions":
                collection = list(validator.context.conditions.conditions.keys())
            elif context_source == "mappings":
                collection = list(validator.context.mappings.maps.keys())
            elif context_source == "refs":
                collection = validator.context.refs

            if collection is not None:
                # Build a dynamic schema with an enum of valid values
                dynamic_schema = {"enum": collection}

                # Use descend to validate against the dynamic schema
                yield from validator.descend(
                    instance=instance,
                    schema=dynamic_schema,
                )

    path_check = dV.get("pathCheck")
    if path_check:
        current_path = "/".join(str(p) for p in validator.context.path.path)
        pattern_schema = {"pattern": f"^{path_check}.*$"}

        yield from validator.descend(
            instance=current_path,
            schema=pattern_schema,
        )


#####
# Type checks
#####
def _raw_type(validator: Validator, tS: Any, instance: Any) -> bool:
    if tS in ["object", "array", "null"] or validator.is_type(instance, "null"):
        return validator.is_type(instance, tS)
    if "string" == tS:
        if validator.is_type(instance, "object") or validator.is_type(
            instance, "array"
        ):
            return False
        return True
    if "number" == tS:
        if validator.is_type(instance, "boolean"):
            return False
        try:
            float(instance)
            return True
        except (ValueError, TypeError):
            return False
    if "integer" == tS:
        if validator.is_type(instance, "boolean"):
            return False
        try:
            int(instance)
            return True
        except (ValueError, TypeError):
            return False
    if "boolean" == tS:
        if validator.is_type(instance, "boolean"):
            return True
        if instance in list(BOOLEAN_STRINGS):
            return True
    return False


def _raw_type_strict(validator: Validator, tS: Any, instance: Any) -> bool:
    return validator.is_type(instance, tS)


# pylint: disable=unused-argument
def cfn_type(validator: Validator, tS: Any, instance: Any, schema: Any):
    """
    When evaluating a type in CloudFormation we have to account
    for the intrinsic functions that the values can represent
    (Ref, GetAtt, If, ...).  This will evaluate if the correct type
    is not found and the instance is an object with a function
    that we do our best to evaluate if that function represents the
    type we are looking for
    """
    if validator.context.strict_types:
        raw_type_fn = _raw_type_strict
    else:
        raw_type_fn = _raw_type
    tS = ensure_list(tS)
    if not any(raw_type_fn(validator, type, instance) for type in tS):
        reprs = ", ".join(repr(type) for type in tS)
        yield ValidationError(f"{instance!r} is not of type {reprs}")


def cfnGather(
    validator: Validator, s: dict[str, Any], instance: Any, schema: dict[str, Any]
) -> ValidationResult:
    """
    Gather properties from local and remote resources, validate with schema.

    Each "gather" entry is local (no "reference") or remote (has "reference").
    The first remote entry drives iteration — its reference is followed to find
    the target resource. All entries' properties are gathered into a flat dict
    keyed by entry name, then validated against "schema" with $data support.
    """
    gather_spec = s.get("gather", {})
    validation_schema = s.get("schema", {})

    # Split entries into local and remote
    local_entries: dict[str, dict[str, Any]] = {}
    remote_entries: dict[str, dict[str, Any]] = {}
    for name, spec in gather_spec.items():
        if "reference" in spec:
            remote_entries[name] = spec
        else:
            local_entries[name] = spec

    if not remote_entries:
        if local_entries:
            gathered, path_maps = _gather_locals(validator, instance, local_entries)
            yield from _validate_gathered(
                validator,
                gathered,
                path_maps,
                {},
                local_entries,
                validation_schema,
            )
        return

    primary_name = next(iter(remote_entries))
    primary_spec = remote_entries[primary_name]
    ref_path = _to_path(primary_spec["reference"])

    for ref_value, scenario_validator in get_value_from_path(
        validator, instance, ref_path
    ):
        if ref_value is None:
            continue

        resource_name = _resolve_reference(ref_value)

        # Unrecognized function — skip
        if resource_name is None:
            if isinstance(ref_value, dict):
                continue
            # Plain value — validate local entries only
            gathered, path_maps = _gather_locals(
                scenario_validator, instance, local_entries
            )
            yield from _validate_gathered(
                scenario_validator,
                gathered,
                path_maps,
                {},
                local_entries,
                validation_schema,
            )
            continue

        if resource_name not in validator.context.resources:
            continue

        target_type = validator.context.resources.get(resource_name, None)

        if not _matches_filter(target_type.type if target_type else None, primary_spec):
            continue

        # Gather all entries
        gathered, path_maps = _gather_locals(
            scenario_validator, instance, local_entries
        )
        resource_names: dict[str, str] = {}

        # Gather all remote entries
        for name, spec in remote_entries.items():
            if name == primary_name:
                rn: str | None = resource_name
                rtype: str | None = target_type.type if target_type else None
            else:
                rn, rtype = _resolve_remote(
                    validator, scenario_validator, instance, spec
                )
            if rn is None:
                continue

            props_spec = spec.get("properties", {})
            gathered[name] = _gather_properties(
                scenario_validator, rn, instance, props_spec, rtype
            )
            path_maps[name] = {k: _prop_path(v) for k, v in props_spec.items()}
            resource_names[name] = rn

        yield from _validate_gathered(
            scenario_validator,
            gathered,
            path_maps,
            resource_names,
            local_entries,
            validation_schema,
        )


def _to_path(path_str: str) -> deque[str | int]:
    if not path_str or path_str == ".":
        return deque()
    return deque(path_str.strip("/").split("/"))


def _prop_path(spec: str | dict[str, Any]) -> list[str]:
    path: str = spec if isinstance(spec, str) else spec.get("path", "")
    return path.strip("/").split("/")


def _resolve_reference(ref_value: Any) -> str | None:
    fn_k, fn_v = is_function(ref_value)
    if fn_k == "Ref" and isinstance(fn_v, str):
        return fn_v
    if fn_k == "Fn::GetAtt":
        if isinstance(fn_v, list) and fn_v:
            return str(fn_v[0])
        if isinstance(fn_v, str):
            return fn_v.split(".")[0]
    return None


def _matches_filter(resource_type: str | None, spec: dict[str, Any]) -> bool:
    target_filter = spec.get("filter", {})
    if target_filter and "type" in target_filter:
        return bool(resource_type == target_filter["type"])
    return True


def _resolve_remote(
    validator: Validator,
    scenario_validator: Validator,
    instance: Any,
    spec: dict[str, Any],
) -> tuple[str | None, str | None]:
    """Follow a remote entry's reference and return (resource_name, type)."""
    ref_path = _to_path(spec["reference"])
    for rv, _ in get_value_from_path(scenario_validator, instance, ref_path):
        if rv is None:
            continue
        rn = _resolve_reference(rv)
        if not rn or rn not in validator.context.resources:
            continue
        rtype = validator.context.resources[rn].type
        if not _matches_filter(rtype, spec):
            continue
        return rn, rtype
    return None, None


def _gather_properties(
    validator: Validator,
    resource_name: str | None,
    instance: Any,
    specs: dict[str, Any],
    resource_type: str | None = None,
) -> dict[str, Any]:
    """Gather property values using get_value_from_path for condition resolution."""
    gathered: dict[str, Any] = {}
    template = validator.cfn.template

    for key, spec in specs.items():
        if isinstance(spec, str) and spec == "$type":
            if resource_type:
                gathered[key] = resource_type
            continue

        if isinstance(spec, str):
            path, default, has_default = spec, None, False
        else:
            path = spec.get("path", "")
            default = spec.get("default")
            has_default = "default" in spec

        path_parts = _to_path(path)

        # Build full path through template for remote, or use instance for local
        if resource_name is not None:
            source = template
            full_path = deque[str | int](["Resources", resource_name, "Properties"])
            full_path.extend(path_parts)
        else:
            source = instance
            full_path = path_parts

        value = None
        found = False
        for val, _ in get_value_from_path(validator, source, full_path.copy()):
            if val is None:
                continue
            fn_k, fn_v = is_function(val)
            if fn_k == "Ref" and fn_v == "AWS::NoValue":
                continue
            value = val
            found = True
            break

        if not found:
            if has_default:
                gathered[key] = default
            continue

        gathered[key] = value
    return gathered


def _gather_locals(
    scenario_validator: Validator,
    instance: Any,
    local_entries: dict[str, dict[str, Any]],
) -> tuple[dict[str, Any], dict[str, dict[str, list[str]]]]:
    """Gather properties for all local entries."""
    gathered: dict[str, Any] = {}
    path_maps: dict[str, dict[str, list[str]]] = {}
    for name, spec in local_entries.items():
        props_spec = spec.get("properties", {})
        gathered[name] = _gather_properties(
            scenario_validator, None, instance, props_spec
        )
        path_maps[name] = {k: _prop_path(v) for k, v in props_spec.items()}
    return gathered, path_maps


def _validate_gathered(
    scenario_validator: Validator,
    gathered: dict[str, Any],
    path_maps: dict[str, dict[str, list[str]]],
    resource_names: dict[str, str],
    local_entries: dict[str, dict[str, Any]],
    validation_schema: dict[str, Any],
) -> ValidationResult:
    """Validate gathered object and remap error paths."""
    resolved_schema = resolve_data_in_schema(validation_schema, gathered)
    for err in scenario_validator.descend(
        gathered, resolved_schema, schema_path="schema"
    ):
        if not err.path:
            yield err
            continue

        err_path = list(err.path)
        entry_name = err_path[0]
        real_path = _remap_path(err_path, path_maps)

        if entry_name in resource_names:
            err.path_override = (
                deque[str | int](
                    ["Resources", resource_names[entry_name], "Properties"]
                )
                + real_path
            )
            err.path = deque()
        elif entry_name in local_entries:
            err.path = real_path
        yield err


def _remap_path(
    err_path: list[str | int],
    path_maps: dict[str, dict[str, list[str]]],
) -> deque[str | int]:
    """Remap a gathered-object error path back to the original property path."""
    entry_name = str(err_path[0])
    if (
        len(err_path) > 1
        and entry_name in path_maps
        and str(err_path[1]) in path_maps[entry_name]
    ):
        real = deque[str | int](path_maps[entry_name][str(err_path[1])])
        real.extend(err_path[2:])
        return real
    return deque(err_path[1:])


def _function_unknown(
    validator: Validator, s: Any, instance: Any, schema: dict[str, Any]
) -> ValidationResult:
    """Default validator for CloudFormation intrinsic functions.
    Returns unknown error in unresolvable mode to prevent incorrect validation.
    Overridden by function rules in full validation environments."""
    if validator.context.unresolvable_function_mode:
        yield ValidationError(
            "Cannot resolve function in composite validation",
            unknown=True,
        )
    # In normal mode, do nothing - let the function pass through
    return


cfn_validators: dict[str, V] = {
    "additionalProperties": additionalProperties,
    "cfnContext": cfnContext,
    "cfnGather": cfnGather,
    "dynamicValidation": dynamicValidation,
    "type": cfn_type,
    # CloudFormation intrinsic functions - default to unknown
    "ref": _function_unknown,
    "fn_base64": _function_unknown,
    "fn_cidr": _function_unknown,
    "fn_findinmap": _function_unknown,
    "fn_getatt": _function_unknown,
    "fn_getazs": _function_unknown,
    "fn_importvalue": _function_unknown,
    "fn_join": _function_unknown,
    "fn_select": _function_unknown,
    "fn_split": _function_unknown,
    "fn_sub": _function_unknown,
    "fn_transform": _function_unknown,
    "fn_tojsonstring": _function_unknown,
    "fn_length": _function_unknown,
    "fn_if": _function_unknown,
}



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any

from cfnlint.jsonschema import ValidationError, Validator
from cfnlint.rules import CloudFormationLintRule


class ResourceLifecycle(CloudFormationLintRule):
    id = "E3710"
    shortdesc = "Resource type is from a service that has been shut down"
    description = (
        "Checks if a resource type belongs to an AWS service that has "
        "reached full shutdown and is no longer available"
    )
    source_url = (
        "https://docs.aws.amazon.com/general/latest/gr/full_shutdown_services.html"
    )
    tags = ["resources", "lifecycle"]

    def __init__(self) -> None:
        super().__init__()
        self.child_rules = {
            "W3696": None,
            "W3697": None,
        }

    _messages = {
        "shutdown": (
            "Resource type {type_name!r} is from a service that has been shut down"
        ),
        "sunset": ("Resource type {type_name!r} is from a service that is sunsetting"),
        "maintenance": (
            "Resource type {type_name!r} is from a service in maintenance mode"
        ),
    }

    _messages_with_date = {
        "shutdown": (
            "Resource type {type_name!r} is from a service that was shut down on {date}"
        ),
        "sunset": (
            "Resource type {type_name!r} is from a service that will "
            "be shut down on {date}. Plan to migrate to an alternative"
        ),
        "maintenance": (
            "Resource type {type_name!r} is from a service in maintenance "
            "mode since {date}. Consider migrating to an alternative"
        ),
    }

    def lifecycle(self, validator: Validator, lc: Any, instance: Any, schema: Any):
        if not isinstance(lc, dict):
            return

        status = lc.get("status", "")
        date = lc.get("date")
        type_name = schema.get("typeName", "")

        if date:
            msg = self._messages_with_date.get(status, "")
            msg = msg.format(type_name=type_name, date=date)
        else:
            msg = self._messages.get(status, "")
            msg = msg.format(type_name=type_name)
        if not msg:
            return

        if status == "shutdown":
            yield ValidationError(msg, rule=self)
        elif status == "sunset":
            rule = self.child_rules.get("W3696")
            if rule:
                yield ValidationError(msg, rule=rule)
        elif status == "maintenance":
            rule = self.child_rules.get("W3697")
            if rule:
                yield ValidationError(msg, rule=rule)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_lambda_permission
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class PermissionPrincipal(CfnLintJsonSchema):
    id = "W3664"
    shortdesc = "Validate Lambda permission Principal matches SourceArn resource type"
    description = (
        "When configuring a Lambda permission with a SourceArn "
        "that references a resource, the Principal should match "
        "the service that owns that resource type"
    )
    source_url = "https://docs.aws.amazon.com/lambda/latest/dg/access-control-resource-based.html"
    tags = ["resources", "lambda", "permission"]

    _type_to_principal: dict[str, str] = {
        "AWS::S3::Bucket": "s3.amazonaws.com",
        "AWS::SNS::Topic": "sns.amazonaws.com",
        "AWS::Events::Rule": "events.amazonaws.com",
        "AWS::ApiGateway::RestApi": "apigateway.amazonaws.com",
        "AWS::ApiGatewayV2::Api": "apigateway.amazonaws.com",
        "AWS::ElasticLoadBalancingV2::TargetGroup": (
            "elasticloadbalancing.amazonaws.com"
        ),
    }

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::Lambda::Permission/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_lambda_permission,
                filename="permission_principal.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any

from cfnlint.helpers import VALID_PARAMETER_TYPES
from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.jsonschema import CfnLintKeyword


class Types(CfnLintKeyword):
    """Check if Parameters are typed"""

    id = "E2002"
    shortdesc = "Parameters have appropriate type"
    description = "Making sure the parameters have a correct type"
    source_url = (
        "https://docs.aws.amazon.com/AWSCloudFormation/latest/"
        "UserGuide/best-practices.html#parmtypes"
    )
    tags = ["parameters"]

    def __init__(self) -> None:
        super().__init__(keywords=["Parameters/*/Type"])

    def validate(
        self, validator: Validator, s: Any, instance: Any, schema: Any
    ) -> ValidationResult:
        if not validator.is_type(instance, "string"):
            return

        # Check if it's a valid documented type
        if instance in VALID_PARAMETER_TYPES:
            return

        # Check if it matches SSM or List pattern (undocumented but accepted)
        if instance.startswith("AWS::SSM::Parameter::Value<") or instance.startswith(
            "List<"
        ):
            return

        # Invalid type
        yield ValidationError(
            f"{instance!r} is not one of {sorted(VALID_PARAMETER_TYPES)!r}",
            rule=self,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_lambda_eventsourcemapping
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class EventSourceMappingToSqsTimeout(CfnLintJsonSchema):
    id = "E3505"
    shortdesc = (
        "Validate SQS 'VisibilityTimeout' is greater than a function's 'Timeout'"
    )
    description = (
        "When attaching a Lambda function to a SQS queue to a Lambda function the "
        "SQS 'VisibilityTimeout' has to be greater than or equal to "
        " the lambda functions's 'Timeout'"
    )
    source_url = "https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html"
    tags = ["resources", "lambda", "sqs"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::Lambda::EventSourceMapping/Properties",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_lambda_eventsourcemapping,
                filename="event_source_sqs_timeout.json",
            ),
            all_matches=True,
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return (
            f"Queue visibility timeout ({err.instance!r}) is less "
            f"than Function timeout ({err.schema.get('minimum')!r}) seconds"
        )

    def _clean_error(self, err: ValidationError) -> ValidationError:
        err = super()._clean_error(err)
        if err.rule == self:
            err.message = self.message(None, err)
        return err



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_sqs_queue
from cfnlint.helpers import bool_compare
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class QueueDLQ(CfnLintJsonSchema):
    id = "E3502"
    shortdesc = "Validate SQS DLQ queues are the same type"
    description = (
        "SQS queues using DLQ have to have the destination "
        "queue as the same type (FIFO or standard)"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sqs-queue.html"
    tags = ["resources", "sqs"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::SQS::Queue/Properties",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_sqs_queue,
                filename="queue_dlq.json",
            ),
            all_matches=True,
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        # Extract the gathered object from the error path
        # The error is on the gathered object, not the original instance
        # Check the expected value from the schema to determine the message
        if err.validator == "const" and "const" in err.schema:
            expected = err.schema["const"]
            # Normalize to boolean for comparison
            if bool_compare(expected, True):
                return (
                    "Source queue type 'standard' does not "
                    "match destination queue type 'FIFO'"
                )
            elif bool_compare(expected, False):
                return (
                    "Source queue type 'FIFO' does not match "
                    "destination queue type 'standard'"
                )
        return self.shortdesc

    def _iter_errors(self, validator, instance):
        """Override to apply custom messages to all errors"""
        errs = list(validator.iter_errors(instance))
        for err in errs:
            err = self._clean_error(err)
            err.message = self.message(instance, err)
            yield err



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_sns_subscription
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class SubscriptionEndpointProtocol(CfnLintJsonSchema):
    id = "W3694"
    shortdesc = "SNS Subscription Endpoint should match Protocol"
    description = (
        "When an SNS Subscription Protocol is 'sqs', the Endpoint "
        "should reference an SQS Queue. When Protocol is 'lambda', "
        "the Endpoint should reference a Lambda Function."
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html"
    tags = ["resources", "sns"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::SNS::Subscription/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_sns_subscription,
                filename="subscription_endpoint_protocol.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_autoscaling_autoscalinggroup
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class AutoScalingMinMaxSize(CfnLintJsonSchema):
    id = "E3706"
    shortdesc = "MaxSize must be greater than or equal to MinSize"
    description = (
        "Validates that AutoScaling group MaxSize is greater than or equal to MinSize"
    )
    source_url = "https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-scaling-simple-step.html"
    tags = ["resources", "autoscaling"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::AutoScaling::AutoScalingGroup/Properties",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_autoscaling_autoscalinggroup,
                filename="min_max_size.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_apigateway_method
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class MethodAuthorizerRestApi(CfnLintJsonSchema):
    id = "E3699"
    shortdesc = "API Gateway Method and Authorizer must use the same RestApi"
    description = (
        "When an API Gateway Method references an Authorizer, both "
        "must reference the same RestApi. A mismatch causes a "
        "deployment failure."
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html"
    tags = ["resources", "apigateway"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::ApiGateway::Method/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_apigateway_method,
                filename="method_authorizer_rest_api.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import cfnlint.data.schemas.extensions.aws_lambda_permission
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class PermissionSourceAccount(CfnLintJsonSchema):
    id = "W3663"
    shortdesc = "Validate SourceAccount is required property"
    description = (
        "When configuration a Lambda permission with a SourceArn "
        "that doesn't have an AccountId you should also specify "
        "the SourceAccount"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-sourceaccount"
    tags = ["resources", "lambda", "permission"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::Lambda::Permission/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_lambda_permission,
                filename="permission_source_account.json",
            ),
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema


class DetectorExclusiveProperties(CfnLintJsonSchema):
    id = "E3063"
    shortdesc = "Validate GuardDuty Detector property exclusivity"
    description = (
        "The request failed because both DataSources and Features were provided. "
        "You can provide only one; it is recommended to use Features."
    )
    source_url = "https://docs.aws.amazon.com/pt_br/guardduty/latest/ug/guardduty-features-activation-model.html"
    tags = ["resources", "guardduty"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::GuardDuty::Detector/Properties",
            ],
            all_matches=True,
        )

    def validate(
        self, validator: Validator, _: Any, instance: Any, schema: dict[str, Any]
    ) -> ValidationResult:
        if "DataSources" in instance and "Features" in instance:
            yield ValidationError(
                "Both 'DataSources' and 'Features' were provided. "
                "You can provide only one; it is recommended to use 'Features'.",
                rule=self,
            )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import cfnlint.data.schemas.extensions.aws_ecs_service
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class ServiceNetworkConfiguration(CfnLintJsonSchema):
    id = "E3052"
    shortdesc = "Validate ECS service requires NetworkConfiguration"
    description = (
        "When using an ECS task definition has NetworkMode set to "
        "'awsvpc' then 'NetworkConfiguration' is required"
    )
    tags = ["resources", "ecs"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::ECS::Service/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_ecs_service,
                filename="service_network_configuration.json",
            ),
            all_matches=True,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_rds_dbinstance
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class DbInstanceClusterStorageEncrypted(CfnLintJsonSchema):
    id = "E3709"
    shortdesc = "Validate RDS DBInstance StorageEncrypted matches DBCluster"
    description = (
        "When a DBInstance references a DBCluster via DBClusterIdentifier, "
        "the StorageEncrypted property must match between the two resources"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbinstance.html"
    tags = ["resources", "rds"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::RDS::DBInstance/Properties",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_rds_dbinstance,
                filename="instance_cluster_storage_encrypted.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message
